"""
Handling Superseded Functionality

The main mechanism in Sage to deal with superseded functionality is to
add a deprecation warning. This will be shown once, the first time
that the deprecated function is called.

Note that all doctests in the following use the trac ticket number
:trac:`13109`, which is where this mandatory argument to
:func:`deprecation` was introduced.

Functions and classes
---------------------
"""



########################################################################
#       Copyright (C) 2012 William Stein <wstein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#                  https://www.gnu.org/licenses/
########################################################################

from warnings import warn
import inspect

from sage.misc.lazy_attribute import lazy_attribute


def _check_trac_number(trac_number):
    """
    Check that the argument is likely to be a valid trac issue number.

    INPUT:

    - ``trac_number`` -- anything.

    OUTPUT:

    This function returns nothing. A ``ValueError`` or ``TypeError`` is
    raised if the argument cannot be a valid trac number.

    EXAMPLES::

        sage: from sage.misc.superseded import _check_trac_number
        sage: _check_trac_number(1)
        sage: _check_trac_number(0)
        Traceback (most recent call last):
        ...
        ValueError: 0 is not a valid trac issue number
        sage: _check_trac_number(int(10))
        sage: _check_trac_number(10.0)
        Traceback (most recent call last):
        ...
        TypeError: 10.0000000000000 is not a valid trac issue number
        sage: _check_trac_number('10')
        Traceback (most recent call last):
        ...
        TypeError: '10' is not a valid trac issue number
    """
    try:
        trac_number = trac_number.__index__()
    except Exception:
        raise TypeError('%r is not a valid trac issue number' % trac_number)
    if trac_number <= 0:
        raise ValueError('%r is not a valid trac issue number' % trac_number)


def deprecation(trac_number, message, stacklevel=4):
    r"""
    Issue a deprecation warning.

    INPUT:

    - ``trac_number`` -- integer. The trac ticket number where the
      deprecation is introduced.

    - ``message`` -- string. An explanation why things are deprecated
      and by what it should be replaced.

    - ``stack_level`` -- (default: ``4``) an integer. This is passed on to
      :func:`warnings.warn`.

    EXAMPLES::

        sage: def foo():
        ....:  sage.misc.superseded.deprecation(13109, 'the function foo is replaced by bar')
        sage: foo()
        doctest:...: DeprecationWarning: the function foo is replaced by bar
        See http://trac.sagemath.org/13109 for details.

    .. SEEALSO::

        :func:`experimental`,
        :func:`warning`.
    """
    warning(trac_number, message, DeprecationWarning, stacklevel)

def deprecation_cython(trac_number, message, stacklevel=3):
    r"""
    Issue a deprecation warning -- for use in cython functions

    TESTS:

    We check that `deprecation_cython` in a cython function generates a warning
    with the same callsite reference as `deprecation` in a python function, whereas
    `deprecation` in a cython function does not::

        sage: cython('''
        ....: from sage.misc.superseded import deprecation_cython, deprecation
        ....: def foo1():
        ....:     deprecation_cython(100,"boo")
        ....: def foo2():
        ....:     deprecation(100,"boo")
        ....: ''')
        sage: def foo3():
        ....:     deprecation(100,"boo")
        sage: if True:  # Execute the three "with" blocks as one doctest
        ....:     with warnings.catch_warnings(record=True) as w1:
        ....:        warnings.simplefilter("always")
        ....:        foo1()
        ....:     with warnings.catch_warnings(record=True) as w2:
        ....:        warnings.simplefilter("always")
        ....:        foo2()
        ....:     with warnings.catch_warnings(record=True) as w3:
        ....:        warnings.simplefilter("always")
        ....:        foo3()
        sage: w1[0].filename == w3[0].filename
        True
        sage: w2[0].filename == w3[0].filename
        False
     """
    warning(trac_number, message, DeprecationWarning, stacklevel)

def warning(trac_number, message, warning_class=Warning, stacklevel=3):
    r"""
    Issue a warning.

    INPUT:

    - ``trac_number`` -- integer. The trac ticket number where the
      deprecation is introduced.

    - ``message`` -- string. An explanation what is going on.

    - ``warning_class`` -- (default: ``Warning``) a class inherited
      from a Python :class:`~exceptions.Warning`.

    - ``stack_level`` -- (default: ``3``) an integer. This is passed on to
      :func:`warnings.warn`.

    EXAMPLES::

        sage: def foo():
        ....:     sage.misc.superseded.warning(
        ....:         99999,
        ....:         'The syntax will change in future.',
        ....:         FutureWarning)
        sage: foo()
        doctest:...: FutureWarning: The syntax will change in future.
        See https://trac.sagemath.org/99999 for details.

    .. SEEALSO::

        :func:`deprecation`,
        :func:`experimental`,
        :class:`exceptions.Warning`.
    """
    _check_trac_number(trac_number)
    message += '\n'
    if trac_number < 24800:  # to avoid changing all previous doctests
        message += 'See http://trac.sagemath.org/'+ str(trac_number) + ' for details.'
    else:
        message += 'See https://trac.sagemath.org/'+ str(trac_number) + ' for details.'

    # Stack level 3 to get the line number of the code which called
    # the deprecated function which called this function.
    warn(message, warning_class, stacklevel)


def experimental_warning(trac_number, message, stacklevel=4):
    r"""
    Issue a warning that the functionality or class is experimental
    and might change in future.

    INPUT:

    - ``trac_number`` -- an integer. The trac ticket number where the
      experimental functionality was introduced.

    - ``message`` -- a string. An explanation what is going on.

    - ``stack_level`` -- (default: ``4``) an integer. This is passed on to
      :func:`warnings.warn`.

    EXAMPLES::

        sage: def foo():
        ....:    sage.misc.superseded.experimental_warning(
        ....:        66666, 'This function is experimental and '
        ....:               'might change in future.')
        sage: foo()
        doctest:...: FutureWarning: This function is experimental and
        might change in future.
        See https://trac.sagemath.org/66666 for details.

    .. SEEALSO::

        :class:`mark_as_experimental`,
        :func:`warning`,
        :func:`deprecation`.
    """
    warning(trac_number, message, FutureWarning, stacklevel)


class experimental(object):
    def __init__(self, trac_number, stacklevel=4):
        """
        A decorator which warns about the experimental/unstable status of
        the decorated class/method/function.

        INPUT:

        - ``trac_number`` -- an integer. The trac ticket number where this
          code was introduced.

        - ``stack_level`` -- (default: ``4``) an integer. This is passed on to
          :func:`warnings.warn`.

        EXAMPLES::

            sage: @sage.misc.superseded.experimental(trac_number=79997)
            ....: def foo(*args, **kwargs):
            ....:     print("{} {}".format(args, kwargs))
            sage: foo(7, what='Hello')
            doctest:...: FutureWarning: This class/method/function is
            marked as experimental. It, its functionality or its
            interface might change without a formal deprecation.
            See https://trac.sagemath.org/79997 for details.
            (7,) {'what': 'Hello'}

        ::

            sage: class bird(SageObject):
            ....:     @sage.misc.superseded.experimental(trac_number=99999)
            ....:     def __init__(self, *args, **kwargs):
            ....:         print("piep {} {}".format(args, kwargs))
            sage: _ = bird(99)
            doctest:...: FutureWarning: This class/method/function is
            marked as experimental. It, its functionality or its
            interface might change without a formal deprecation.
            See https://trac.sagemath.org/99999 for details.
            piep (99,) {}

        TESTS:

        The following test works together with the doc-test for
        :meth:`__experimental_self_test` to demonstrate that warnings are issued only
        once, even in doc-tests (see :trac:`20601`).
        ::

            sage: from sage.misc.superseded import __experimental_self_test
            sage: _ = __experimental_self_test("A")
            doctest:...: FutureWarning: This class/method/function is
            marked as experimental. It, its functionality or its
            interface might change without a formal deprecation.
            See https://trac.sagemath.org/88888 for details.
            I'm A

        .. SEEALSO::

            :func:`experimental`,
            :func:`warning`,
            :func:`deprecation`.
        """
        self.trac_number = trac_number
        self.stacklevel = stacklevel

    def __call__(self, func):
        """
        Print experimental warning.

        INPUT:

        - ``func`` -- the function to decorate.

        OUTPUT:

        The wrapper to this function.

        TESTS::

            sage: def foo(*args, **kwargs):
            ....:     print("{} {}".format(args, kwargs))
            sage: from sage.misc.superseded import experimental
            sage: ex_foo = experimental(trac_number=99399)(foo)
            sage: ex_foo(3, what='Hello')
            doctest:...: FutureWarning: This class/method/function is
            marked as experimental. It, its functionality or its
            interface might change without a formal deprecation.
            See https://trac.sagemath.org/99399 for details.
            (3,) {'what': 'Hello'}
        """
        from sage.misc.decorators import sage_wraps
        @sage_wraps(func)
        def wrapper(*args, **kwds):
            if not wrapper._already_issued:
                experimental_warning(self.trac_number,
                            'This class/method/function is marked as '
                            'experimental. It, its functionality or its '
                            'interface might change without a '
                            'formal deprecation.',
                            self.stacklevel)
                wrapper._already_issued = True
            return func(*args, **kwds)
        wrapper._already_issued = False

        return wrapper


class __experimental_self_test(object):
    r"""
    This is a class only to demonstrate with a doc-test that the @experimental
    decorator only issues a warning message once (see :trac:`20601`).

    The test below does not issue a warning message because that warning has
    already been issued by a previous doc-test in the @experimental code. Note
    that this behaviour cannot be demonstrated within a single documentation
    string: Sphinx will itself supress multiple issued warnings.

    TESTS::

        sage: from sage.misc.superseded import __experimental_self_test
        sage: _ = __experimental_self_test("B")
        I'm B
    """
    @experimental(trac_number=88888)
    def __init__(self, x):
        print("I'm " + x)


class DeprecatedFunctionAlias(object):
    """
    A wrapper around methods or functions which automatically prints a
    deprecation message. See :func:`deprecated_function_alias`.

    AUTHORS:

    - Florent Hivert (2009-11-23), with the help of Mike Hansen.
    - Luca De Feo (2011-07-11), printing the full module path when different from old path
    """
    def __init__(self, trac_number, func, module, instance=None, unbound=None):
        r"""
        TESTS::

            sage: from sage.misc.superseded import deprecated_function_alias
            sage: g = deprecated_function_alias(13109, number_of_partitions)
            sage: from sage.misc.superseded import deprecated_function_alias
            sage: g.__doc__
            'Deprecated: Use :func:`number_of_partitions` instead.\nSee :trac:`13109` for details.\n\n'
        """
        _check_trac_number(trac_number)
        try:
            self.__dict__.update(func.__dict__)
        except AttributeError:
            pass # Cython classes don't have __dict__
        self.func = func
        self.trac_number  = trac_number
        self.instance = instance # for use with methods
        self.unbound = unbound
        self.__module__ = module
        if isinstance(func, type(deprecation)):
            sphinxrole = "func"
        else:
            sphinxrole = "meth"
        doc = 'Deprecated: '
        doc += 'Use :' + sphinxrole + ':`' + self.func.__name__ + '` instead.\n'
        doc += 'See :trac:`' + str(self.trac_number) + '` for details.\n\n'
        self.__doc__ = doc

    @lazy_attribute
    def __name__(self):
        """
        TESTS::

            sage: from sage.misc.superseded import deprecated_function_alias
            sage: g = deprecated_function_alias(13109, number_of_partitions)
            sage: g.__name__
            'g'

            sage: from sage.misc.superseded import deprecated_function_alias
            sage: class cls(object):
            ....:    def new_meth(self): return 42
            ....:    old_meth = deprecated_function_alias(13109, new_meth)
            sage: cls.old_meth.__name__
            'old_meth'
            sage: cls().old_meth.__name__
            'old_meth'

            sage: cython('\n'.join([
            ....:     r"from sage.misc.superseded import deprecated_function_alias",
            ....:     r"cdef class cython_cls(object):",
            ....:     r"    def new_cython_meth(self):",
            ....:     r"        return 1",
            ....:     r"    old_cython_meth = deprecated_function_alias(13109, new_cython_meth)"
            ....: ]))
            sage: cython_cls().old_cython_meth.__name__
            'old_cython_meth'
        """
        # first look through variables in stack frames
        for frame in inspect.stack():
            for name, obj in frame[0].f_globals.items():
                if obj is self:
                    return name
        # then search object that contains self as method
        import gc
        import copy
        gc.collect()

        def is_class(gc_ref):
            if not isinstance(gc_ref, dict):
                return False
            is_python_class = '__module__' in gc_ref or '__package__' in gc_ref
            is_cython_class = '__new__' in gc_ref
            return is_python_class or is_cython_class
        search_for = self if (self.unbound is None) else self.unbound
        for ref in gc.get_referrers(search_for):
            if is_class(ref) and ref is not self.__dict__:
                ref_copy = copy.copy(ref)
                for key, val in ref_copy.items():
                    if val is search_for:
                        return key
        raise AttributeError("The name of this deprecated function cannot be determined")

    def __call__(self, *args, **kwds):
        """
        TESTS::

            sage: from sage.misc.superseded import deprecated_function_alias
            sage: def bla(): return 42
            sage: blo = deprecated_function_alias(13109, bla)
            sage: blo()
            doctest:...: DeprecationWarning: blo is deprecated. Please use bla instead.
            See http://trac.sagemath.org/13109 for details.
            42
        """
        if self.instance is None and self.__module__ != self.func.__module__:
            other = self.func.__module__ + "." + self.func.__name__
        else:
            other = self.func.__name__

        deprecation(self.trac_number,
                    "{} is deprecated. Please use {} instead.".format(
                        self.__name__, other))
        if self.instance is None:
            return self.func(*args, **kwds)
        else:
            return self.func(self.instance, *args, **kwds)

    def __get__(self, inst, cls=None):
        """
        TESTS::

            sage: from sage.misc.superseded import deprecated_function_alias
            sage: class cls(object):
            ....:    def new_meth(self): return 42
            ....:    old_meth = deprecated_function_alias(13109, new_meth)
            sage: obj = cls()
            sage: obj.old_meth.instance is obj
            True

        :trac:`19125`::

            sage: from sage.misc.superseded import deprecated_function_alias
            sage: class A:
            ....:    def __init__(self, x):
            ....:        self.x = x
            ....:    def f(self, y):
            ....:        return self.x+y
            ....:    g = deprecated_function_alias(42, f)
            sage: a1 = A(1)
            sage: a2 = A(2)
            sage: a1.g(a2.g(0))
            doctest:...: DeprecationWarning: g is deprecated. Please use f instead.
            See http://trac.sagemath.org/42 for details.
            3
            sage: a1.f(a2.f(0))
            3

        """
        if inst is None:
            return self  # Unbound method lookup on class
        else:
            # Return a bound method wrapper
            return DeprecatedFunctionAlias(self.trac_number, self.func,
                                           self.__module__, instance=inst,
                                           unbound=self)


def deprecated_function_alias(trac_number, func):
    """
    Create an aliased version of a function or a method which raises a
    deprecation warning message.

    If f is a function or a method, write
    ``g = deprecated_function_alias(trac_number, f)``
    to make a deprecated aliased version of f.

    INPUT:

    - ``trac_number`` -- integer. The trac ticket number where the
      deprecation is introduced.

    - ``func`` -- the function or method to be aliased

    EXAMPLES::

        sage: from sage.misc.superseded import deprecated_function_alias
        sage: g = deprecated_function_alias(13109, number_of_partitions)
        sage: g(5)
        doctest:...: DeprecationWarning: g is deprecated. Please use sage.combinat.partition.number_of_partitions instead.
        See http://trac.sagemath.org/13109 for details.
        7

    This also works for methods::

        sage: class cls(object):
        ....:    def new_meth(self): return 42
        ....:    old_meth = deprecated_function_alias(13109, new_meth)
        sage: cls().old_meth()
        doctest:...: DeprecationWarning: old_meth is deprecated. Please use new_meth instead.
        See http://trac.sagemath.org/13109 for details.
        42

    :trac:`11585`::

        sage: def a(): pass
        sage: b = deprecated_function_alias(13109, a)
        sage: b()
        doctest:...: DeprecationWarning: b is deprecated. Please use a instead.
        See http://trac.sagemath.org/13109 for details.

    AUTHORS:

     - Florent Hivert (2009-11-23), with the help of Mike Hansen.
     - Luca De Feo (2011-07-11), printing the full module path when different from old path
    """
    module_name = None
    frame0 = inspect.currentframe()
    if frame0:
        frame1 = frame0.f_back
        if frame1:
            module_name = inspect.getmodulename(frame1.f_code.co_filename)
    if module_name is None:
        module_name = '__main__'
    return DeprecatedFunctionAlias(trac_number, func, module_name)



r"""
Shephard Groups
"""
#*****************************************************************************
#  Copyright (C) 2016 Travis Scrimshaw <tscrim at ucdavis.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from sage.categories.category_singleton import Category_singleton
from sage.categories.generalized_coxeter_groups import GeneralizedCoxeterGroups
from sage.misc.cachefunc import cached_method

class ShephardGroups(Category_singleton):
    r"""
    The category of Shephard groups.

    EXAMPLES::

        sage: from sage.categories.shephard_groups import ShephardGroups
        sage: C = ShephardGroups(); C
        Category of shephard groups

    TESTS::

        sage: TestSuite(C).run()
    """
    @cached_method
    def super_categories(self):
        """
        EXAMPLES::

            sage: from sage.categories.shephard_groups import ShephardGroups
            sage: ShephardGroups().super_categories()
            [Category of finite generalized coxeter groups]
        """
        return [GeneralizedCoxeterGroups().Finite()]




"""
Magics for each of the Sage interfaces

This module defines magic functions for interpreters. As an example,
consider the GAP interpreter which can evaluate a gap command given as
a string::

    sage: gap('SymmetricGroup(4)')        # not tested
    SymmetricGroup( [ 1 .. 4 ] )

Magics are syntactic sugar to avoid writing the Python string. They
are either called as line magics::

    sage: %gap SymmetricGroup(4)          # not tested

or as cell magics, that is, spanning multiple lines::

    sage: %%gap                           # not tested
    ....: G := SymmetricGroup(4);
    ....: Display(G);

Note that the cell magic needs semicolons, this is required by the GAP
language to separate multiple commands.
"""
# Note: no magics in doctests, hence # not tested


# ****************************************************************************
#       Copyright (C) 2016 Volker Braun <vbraun.name@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************


from sage.repl.rich_output.display_manager import get_display_manager


LINE_DOCSTRING = """
Interact with {name}

The line magic %{name} sends a single line to the {name} interface.

INPUT:

Single {name} command

OUTPUT:

The result of the evaluated {name} command as an interface wrapper
object.

EXAMPLES::

    sage: %{name} 1 + 2 + 3     # not tested
"""


CELL_DOCSTRING = """
Interact with {name}

The cell magic %%{name} sends multiple lines to the {name} interface.

INPUT:

Multiple lines of valid {name}-commands

OUTPUT:

The result of the evaluated {name}-commands is printed.

EXAMPLES::

    sage: %%{name}                    # not tested
    ....: 1 + 2 + 3;
    ....: some_{name}_command();
"""


class InterfaceMagic(object):

    @classmethod
    def all_iter(cls):
        """
        Iterate over the available interfaces

        EXAMPLES::

            sage: from sage.repl.interface_magic import InterfaceMagic
            sage: next(InterfaceMagic.all_iter())
            <sage.repl.interface_magic.InterfaceMagic object at 0x...>
        """
        import sage.interfaces.all
        for name, obj in sage.interfaces.all.__dict__.items():
            if isinstance(obj, sage.interfaces.interface.Interface):
                yield cls(name, obj)

    @classmethod
    def register_all(cls, shell=None):
        """
        Register all available interfaces

        EXAMPLES::

            sage: class MockShell():
            ....:     magics = set()
            ....:     def register_magic_function(self, fn, magic_name, magic_kind):
            ....:         self.magics.add(magic_name)
            ....:         print(magic_name, magic_kind)
            sage: from sage.repl.interface_magic import InterfaceMagic
            sage: InterfaceMagic.register_all(MockShell())    # random output
            ('gp', 'line')
            ('gp', 'cell')
            ('mwrank', 'line')
            ('mwrank', 'cell')
            ...
            ('maxima', 'line')
            ('maxima', 'cell')
            sage: 'gap' in MockShell.magics
            True
            sage: 'maxima' in MockShell.magics
            True
        """
        if shell is None:
            shell = get_ipython()
        for interface in cls.all_iter():
            shell.register_magic_function(
                interface.line_magic_factory(),
                magic_name=interface._name,
                magic_kind='line'
            )
            shell.register_magic_function(
                interface.cell_magic_factory(),
                magic_name=interface._name,
                magic_kind='cell'
            )

    @classmethod
    def find(cls, name):
        """
        Find a particular magic by name

        This method is for doctesting purposes only.

        INPUT:

        - ``name`` -- string. The name of the interface magic to
          search for.

        OUTPUT:

        The corresponding :class:`InterfaceMagic` instance.

        EXAMPLES::

            sage: from sage.repl.interface_magic import InterfaceMagic
            sage: InterfaceMagic.find('gap')
            <sage.repl.interface_magic.InterfaceMagic object at 0x...>
        """
        for magic in cls.all_iter():
            if magic._name == name:
                return magic

    def __init__(self, name, interface):
        """
        Interface Magic

        This class is a wrapper around interface objects to provide
        them with magics.

        INPUT:

        - ``name`` -- string. The interface name

        - ``interface`` -- :class:`sage.interfaces.expect.Expect`. The
          interface to wrap.

        EXAMPLES::

            sage: from sage.repl.interface_magic import InterfaceMagic
            sage: InterfaceMagic.find('gap')
            <sage.repl.interface_magic.InterfaceMagic object at 0x...>
        """
        self._name = name
        self._interface = interface

    def line_magic_factory(self):
        """
        Factory for line magic

        OUTPUT:

        A function suitable to be used as line magic.

        EXAMPLES::

            sage: from sage.repl.interface_magic import InterfaceMagic
            sage: line_magic = InterfaceMagic.find('gap').line_magic_factory()
            sage: output = line_magic('1+1')
            sage: output
            2
            sage: type(output)
            <class 'sage.interfaces.gap.GapElement'>

        This is how the built line magic is used in practice::

            sage: from sage.repl.interpreter import get_test_shell
            sage: shell = get_test_shell()
            sage: shell.run_cell('%gap 1+1')
            2
            sage: shell.run_cell('%gap?')
            Docstring:
            Interact with gap
            <BLANKLINE>
            The line magic %gap sends a single line to the gap interface.
            ...
        """
        terminal = get_display_manager().is_in_terminal()

        def line_magic(line):
            if line:
                return self._interface(line)
            else:
                if terminal:
                    self._interface.interact()
                else:
                    raise SyntaxError('{0} command required'.format(self._name))
        line_magic.__doc__ = LINE_DOCSTRING.format(name=self._name)
        return line_magic

    def cell_magic_factory(self):
        r"""
        Factory for cell magic

        OUTPUT:

        A function suitable to be used as cell magic.

        EXAMPLES::

            sage: from sage.repl.interface_magic import InterfaceMagic
            sage: cell_magic = InterfaceMagic.find('gap').cell_magic_factory()
            sage: output = cell_magic('', '1+1;')
            2
            sage: output is None
            True
            sage: cell_magic('foo', '1+1;')
            Traceback (most recent call last):
            ...
            SyntaxError: Interface magics have no options, got "foo"

        This is how the built cell magic is used in practice::

            sage: from sage.repl.interpreter import get_test_shell
            sage: shell = get_test_shell()
            sage: shell.run_cell('%%gap\nG:=SymmetricGroup(5);\n1+1;Order(G);')
            Sym( [ 1 .. 5 ] )
            2
            120
            sage: shell.run_cell('%%gap foo\n1+1;\n')
            ...File "<string>", line unknown
            SyntaxError: Interface magics have no options, got "foo"
            <BLANKLINE>
            sage: shell.run_cell('%%gap?')
            Docstring:
            Interact with gap
            <BLANKLINE>
            The cell magic %%gap sends multiple lines to the gap interface.
            ...
        """
        def cell_magic(line, cell):
            """
            Evaluate cell magic

            Docstring is overwritten in the instance

            INPUT:

            - ``line`` -- string. The option part of the cell magic.

            - ``cell`` -- string. The lines of the cell magic.

            OUTPUT:

            Prints the interface output.

            RAISES:

            ``SyntaxError`` if a line is specified; Interfaces have no
            options.
            """
            if line:
                raise SyntaxError('Interface magics have no options, got "{0}"'.format(line))
            output = self._interface.eval(cell)
            print(output)
        cell_magic.__doc__ = CELL_DOCSTRING.format(name=self._name)
        return cell_magic



r"""
Gcd domains
"""
#*****************************************************************************
#  Copyright (C) 2008 Teresa Gomez-Diaz (CNRS) <Teresa.Gomez-Diaz@univ-mlv.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.categories.category_singleton import Category_singleton
from sage.categories.integral_domains import IntegralDomains

class GcdDomains(Category_singleton):
    """
    The category of gcd domains
    domains where gcd can be computed but where there is no guarantee of
    factorisation into irreducibles

    EXAMPLES::

        sage: GcdDomains()
        Category of gcd domains
        sage: GcdDomains().super_categories()
        [Category of integral domains]

    TESTS::

        sage: TestSuite(GcdDomains()).run()
    """

    def super_categories(self):
        """
        EXAMPLES::

            sage: GcdDomains().super_categories()
            [Category of integral domains]
        """
        return [IntegralDomains()]

    def additional_structure(self):
        """
        Return ``None``.

        Indeed, the category of gcd domains defines no additional
        structure: a ring morphism between two gcd domains is a gcd
        domain morphism.

        .. SEEALSO:: :meth:`Category.additional_structure`

        EXAMPLES::

            sage: GcdDomains().additional_structure()
        """
        return None

    class ParentMethods:
        pass

    class ElementMethods:
        # gcd(x,y)
        # lcm(x,y)
        pass



#*****************************************************************************
#  Copyright (C) 2012 Florent Hivert <Florent.Hivert at lri.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from sage.misc.nested_class cimport NestedClassMetaclass

cdef class ClasscallMetaclass(NestedClassMetaclass):
    cdef object classcall
    cdef object classget
    cdef object classcontains



"""
Fast methods via Cython

This module provides extension classes with useful methods of cython speed,
that python classes can inherit.

.. NOTE::

    This module provides a cython base class :class:`WithEqualityById`
    implementing unique instance behaviour, and a cython base class
    :class:`FastHashable_class`, which has a quite fast hash
    whose value can be freely chosen at initialisation time.

AUTHOR:

- Simon King (2013-02): Original version
- Simon King (2013-10): Add :class:`Singleton`

"""

#*****************************************************************************
#       Copyright (C) 2013 Simon A. King <simon.king at uni-jena.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from sage.misc.classcall_metaclass import ClasscallMetaclass, typecall
from sage.misc.constant_function import ConstantFunction

from cpython.object cimport Py_EQ, Py_NE


cdef class WithEqualityById:
    """
    Provide hash and equality test based on identity.

    .. NOTE::

        This class provides the unique representation behaviour of
        :class:`~sage.structure.unique_representation.UniqueRepresentation`,
        together with :class:`~sage.structure.unique_representation.CachedRepresentation`.

    EXAMPLES:

    Any instance of :class:`~sage.structure.unique_representation.UniqueRepresentation`
    inherits from :class:`WithEqualityById`.
    ::

        sage: class MyParent(Parent):
        ....:   def __init__(self, x):
        ....:       self.x = x
        ....:   def __hash__(self):
        ....:       return hash(self.x)
        sage: class MyUniqueParent(UniqueRepresentation, MyParent): pass
        sage: issubclass(MyUniqueParent, sage.misc.fast_methods.WithEqualityById)
        True

    Inheriting from :class:`WithEqualityById` provides unique representation
    behaviour::

        sage: a = MyUniqueParent(1)
        sage: b = MyUniqueParent(2)
        sage: c = MyUniqueParent(1)
        sage: a is c
        True
        sage: d = MyUniqueParent(-1)
        sage: a == d
        False

    The hash inherited from ``MyParent`` is replaced by a hash that coincides
    with :class:`object`'s hash::

        sage: hash(a) == hash(a.x)
        False
        sage: hash(a) == object.__hash__(a)
        True

    .. WARNING::

        It is possible to inherit from
        :class:`~sage.structure.unique_representation.UniqueRepresentation`
        and then overload equality test in a way that destroys the unique
        representation property. We strongly recommend against it!  You should
        use :class:`~sage.structure.unique_representation.CachedRepresentation`
        instead.

    ::

        sage: class MyNonUniqueParent(MyUniqueParent):
        ....:   def __eq__(self, other):
        ....:       return self.x^2 == other.x^2
        sage: a = MyNonUniqueParent(1)
        sage: d = MyNonUniqueParent(-1)
        sage: a is MyNonUniqueParent(1)
        True
        sage: a == d
        True
        sage: a is d
        False

    """
    def __hash__(self):
        """
        The hash provided by this class coincides with that of ``<class 'object'>``.

        TESTS::

            sage: class MyParent(Parent):
            ....:   def __init__(self, x):
            ....:       self.x = x
            ....:   def __hash__(self):
            ....:       return hash(self.x)
            sage: class MyUniqueParent(UniqueRepresentation, MyParent): pass
            sage: issubclass(MyUniqueParent, sage.misc.fast_methods.WithEqualityById)
            True
            sage: a = MyUniqueParent(1)
            sage: hash(a) == hash(a.x)
            False
            sage: hash(a) == object.__hash__(a)
            True

            sage: from sage.misc.fast_methods import WithEqualityById
            sage: o1 = WithEqualityById()
            sage: o2 = WithEqualityById()
            sage: hash(o1) == hash(o2)
            False
        """
        # This is the default hash function in Python's object.c:
        return hash_by_id(<void *>self)

    def __richcmp__(self, other, int op):
        """
        Equality test provided by this class is by identity.

        TESTS::

            sage: class MyParent(Parent):
            ....:   def __init__(self, x):
            ....:       self.x = x
            ....:   def __hash__(self):
            ....:       return hash(self.x)
            sage: class MyUniqueParent(UniqueRepresentation, MyParent): pass
            sage: issubclass(MyUniqueParent, sage.misc.fast_methods.WithEqualityById)
            True
            sage: a = MyUniqueParent(1)
            sage: b = MyUniqueParent(-1)

        Equality test takes into account identity::

            sage: a == b
            False

        When comparing with an object which is not an instance of
        ``WithEqualityById``, the other object determines the
        comparison::

            sage: class AlwaysEqual:
            ....:     def __eq__(self, other):
            ....:         return True
            sage: AlwaysEqual() == a
            True
            sage: a == AlwaysEqual()
            True

        Check that :trac:`19628` is fixed::

            sage: from sage.misc.lazy_import import LazyImport
            sage: lazyQQ = LazyImport('sage.all', 'QQ')
            sage: PolynomialRing(lazyQQ, 'ijk') is PolynomialRing(QQ, 'ijk')
            True
            sage: PolynomialRing(QQ, 'ijkl') is PolynomialRing(lazyQQ, 'ijkl')
            True
        """
        # This only makes sense if "other" is also of type WithEqualityById
        if type(self) is not type(other):
            if not isinstance(other, WithEqualityById):
                return NotImplemented

        if op == Py_EQ:
            return self is other
        elif op == Py_NE:
            return self is not other
        return NotImplemented


cdef class FastHashable_class:
    """
    A class that has a fast hash method, returning a pre-assigned value.

    .. NOTE::

        This is for internal use only. The class has a cdef attribute
        ``_hash``, that needs to be assigned (for example, by calling
        the init method, or by a direct assignement using
        cython). This is slower than using :func:`provide_hash_by_id`,
        but has the advantage that the hash can be prescribed, by
        assigning a cdef attribute ``_hash``.

    TESTS::

        sage: from sage.misc.fast_methods import FastHashable_class
        sage: H = FastHashable_class(123)
        sage: hash(H)
        123
    """
    def __init__(self, h):
        """
        TESTS::

            sage: from sage.misc.fast_methods import FastHashable_class
            sage: H = FastHashable_class(123)
            sage: hash(H)   # indirect doctest
            123
        """
        self._hash = h

    def __hash__(self):
        """
        TESTS::

            sage: from sage.misc.fast_methods import FastHashable_class
            sage: H = FastHashable_class(123)
            sage: hash(H)   # indirect doctest
            123

        """
        return self._hash


class Singleton(WithEqualityById, metaclass=ClasscallMetaclass):
    """
    A base class for singletons.

    A singleton is a class that allows to create not more than a
    single instance. This instance can also belong to a subclass, but
    it is not possible to have several subclasses of a singleton all
    having distinct unique instances.

    In order to create a singleton, just add :class:`Singleton`
    to the list of base classes::

        sage: from sage.misc.fast_methods import Singleton
        sage: class C(Singleton, SageObject):
        ....:     def __init__(self):
        ....:         print("creating singleton")
        sage: c = C()
        creating singleton
        sage: c2 = C()
        sage: c is c2
        True

    The unique instance of a singleton stays in memory as long as the
    singleton itself does.

    Pickling, copying, hashing, and comparison are provided for by
    :class:`Singleton` according to the singleton paradigm. Note
    that pickling fails if the class is replaced by a sub-sub-class
    after creation of the instance::

        sage: class D(C):
        ....:     pass
        sage: import __main__      # This is only needed ...
        sage: __main__.C = C       # ... in doctests
        sage: __main__.D = D       # same here, only in doctests
        sage: orig = type(c)
        sage: c.__class__ = D
        sage: orig == type(c)
        False
        sage: loads(dumps(c))
        Traceback (most recent call last):
        ...
        AssertionError: <class '__main__.D'> is not a direct subclass of <class 'sage.misc.fast_methods.Singleton'>
    """
    @staticmethod
    def __classcall__(cls):
        """
        Create an instance ``O`` of the given class ``cls``, and make it
        so that in future both ``cls.__call__`` and ``O.__class__.__call__``
        are constant functions returning ``O``.

        EXAMPLES::

            sage: from sage.misc.fast_methods import Singleton
            sage: class C(Singleton, Parent):
            ....:     def __init__(self):
            ....:         print("creating singleton")
            ....:         Parent.__init__(self, base=ZZ, category=Rings())
            sage: c = C()
            creating singleton
            sage: import __main__      # This is only needed ...
            sage: __main__.C = C       # ... in doctests
            sage: loads(dumps(c)) is copy(c) is C()  # indirect doctest
            True
        """
        assert cls.mro()[1] == Singleton, "{} is not a direct subclass of {}".format(cls, Singleton)
        res = typecall(cls)
        cf = ConstantFunction(res)
        cls._set_classcall(cf)
        res.__class__._set_classcall(cf)
        return res

    def __copy__(self):
        """
        There is a unique instance of a singleton, hence, copying
        returns ``self``.

        EXAMPLES::

            sage: from sage.misc.fast_methods import Singleton
            sage: class C(Singleton, Parent):                  
            ....:     def __init__(self):
            ....:         print("creating singleton")
            ....:         Parent.__init__(self, base=ZZ, category=Rings())
            sage: c = C()
            creating singleton
            sage: import __main__      # This is only needed ...
            sage: __main__.C = C       # ... in doctests
            sage: loads(dumps(c)) is copy(c) is C()  # indirect doctest
            True
        """ 
        return self

    def __reduce__(self):
        """
        There is a unique instance of a singleton, hence, pickling
        returns ``self``.

        EXAMPLES::

            sage: from sage.misc.fast_methods import Singleton
            sage: class C(Singleton, Parent):                  
            ....:     def __init__(self):
            ....:         print("creating singleton")
            ....:         Parent.__init__(self, base=ZZ, category=Rings())
            ....:
            sage: c = C()
            creating singleton
            sage: import __main__      # This is only needed ...
            sage: __main__.C = C       # ... in doctests
            sage: loads(dumps(c)) is copy(c) is C()  # indirect doctest
            True
 
        The pickle data mainly consist of the class of the unique instance,
        which may be a subclass of the original class used to create the
        instance.If the class is replaced by a sub-sub-class after creation
        of the instance, pickling fails. See the doctest
        in :class:`Singleton`.
        """ 
        return self.__class__, ()



r"""
Special Methods for Classes

AUTHORS:

- Nicolas M. Thiery (2009-2011) implementation of
  ``__classcall__``, ``__classget__``, ``__classcontains__``;
- Florent Hivert (2010-2012): implementation of ``__classcall_private__``,
  documentation, Cythonization and optimization.
"""

# ****************************************************************************
#       Copyright (C) 2009      Nicolas M. Thiery <nthiery at users.sf.net>
#       Copyright (C) 2010-2012 Florent Hivert <Florent.Hivert at lri.fr>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from cpython.object cimport *
from cpython.type cimport type as pytype

__all__ = ['ClasscallMetaclass', 'typecall', 'timeCall']


cdef class ClasscallMetaclass(NestedClassMetaclass):
    r"""
    A metaclass providing support for special methods for classes.

    From the Section :python:`Special method names
    <reference/datamodel.html#special-method-names>` of the Python Reference
    Manual:

        \`a class ``cls`` can implement certain operations on its instances
        that are invoked by special syntax (such as arithmetic operations or
        subscripting and slicing) by defining methods with special
        names\'.

    The purpose of this metaclass is to allow for the class ``cls`` to
    implement analogues of those special methods for the operations on the
    class itself.

    Currently, the following special methods are supported:

     - ``.__classcall__`` (and ``.__classcall_private__``) for
       customizing ``cls(...)`` (analogue of ``.__call__``).

     - ``.__classcontains__`` for customizing membership testing
       ``x in cls`` (analogue of ``.__contains__``).

     - ``.__classget__`` for customizing the binding behavior in
       ``foo.cls`` (analogue of ``.__get__``).

    See the documentation of :meth:`__call__` and of :meth:`__get__`
    and :meth:`__contains__` for the description of the respective
    protocols.

    .. WARNING::

        For technical reasons, ``__classcall__``,
        ``__classcall_private__``, ``__classcontains__``, and
        ``__classget__`` must be defined as :func:`staticmethod`'s,
        even though they receive the class itself as their first
        argument.

    .. WARNING::

        For efficiency reasons, the resolution for the special methods
        is done once for all, upon creation of the class. Thus, later
        dynamic changes to those methods are ignored. But see also
        :meth:`_set_classcall`.

    ``ClasscallMetaclass`` is an extension of the base :class:`type`.

    .. TODO:: find a good name for this metaclass.

    TESTS::

        sage: PerfectMatchings(2).list()
        [[(1, 2)]]

    .. NOTE::

        If a class is put in this metaclass it automatically becomes a
        new-style class::

            sage: from sage.misc.classcall_metaclass import ClasscallMetaclass
            sage: class Foo(metaclass=ClasscallMetaclass): pass
            sage: x = Foo(); x
            <__main__.Foo object at 0x...>
            sage: issubclass(Foo, object)
            True
            sage: isinstance(Foo, type)
            True
    """

    def __cinit__(self, *args, **opts):
        r"""
        TESTS::

            sage: from sage.misc.classcall_metaclass import ClasscallMetaclass
            sage: class FOO(metaclass=ClasscallMetaclass): pass
            sage: isinstance(FOO, ClasscallMetaclass)  # indirect doctest
            True
        """
        if '__classcall_private__' in self.__dict__:
            self.classcall = self.__classcall_private__
        elif hasattr(self, "__classcall__"):
            self.classcall = self.__classcall__
        else:
            self.classcall = None

        self.classcontains = getattr(self, "__classcontains__", None)
        self.classget = getattr(self, "__classget__", None)

    def _set_classcall(cls, function):
        r"""
        Change dynamically the classcall function for this class

        EXAMPLES::

            sage: from sage.misc.classcall_metaclass import ClasscallMetaclass
            sage: class FOO(metaclass=ClasscallMetaclass): pass
            sage: FOO()
            <__main__.FOO object at ...>

        For efficiency reason, the resolution of the ``__classcall__``
        method is done once for all, upon creation of the class. Thus,
        later dynamic changes to this method are ignored by FOO::

            sage: FOO.__classcall__ = ConstantFunction(1)
            sage: FOO()
            <__main__.FOO object at ...>

        but not by subclasses created later on::

            sage: class BAR(FOO): pass
            sage: BAR()
            1

        To update the ``classcall`` special function for FOO, one
        should use this setter::

            sage: FOO._set_classcall(ConstantFunction(2))
            sage: FOO()
            2

        Note that it has no influence on subclasses::

            sage: class BAR(FOO): pass
            sage: BAR()
            1
        """
        cls.classcall = function

    def __call__(cls, *args, **kwds):
        r"""
        This method implements ``cls(<some arguments>)``.

        Let ``cls`` be a class in :class:`ClasscallMetaclass`, and
        consider a call of the form::

            cls(<some arguments>)

        - If ``cls`` defines a method ``__classcall_private__``, then
          this results in a call to::

            cls.__classcall_private__(cls, <some arguments>)

        - Otherwise, if ``cls`` has a method ``__classcall__``, then instead
          the following is called::

            cls.__classcall__(cls, <some arguments>)

        - If neither of these two methods are implemented, then the standard
          ``type.__call__(cls, <some arguments>)`` is called, which in turn
          uses :meth:`~object.__new__` and :meth:`~object.__init__` as usual
          (see Section :python:`Basic Customization
          <reference/datamodel.html#basic-customization>` in the Python
          Reference Manual).

        .. warning:: for technical reasons, ``__classcall__`` must be
            defined as a :func:`staticmethod`, even though it receives
            the class itself as its first argument.

        EXAMPLES::

            sage: from sage.misc.classcall_metaclass import ClasscallMetaclass
            sage: class Foo(metaclass=ClasscallMetaclass):
            ....:     @staticmethod
            ....:     def __classcall__(cls):
            ....:         print("calling classcall")
            ....:         return type.__call__(cls)
            ....:     def __new__(cls):
            ....:         print("calling new")
            ....:         return super(Foo, cls).__new__(cls)
            ....:     def __init__(self):
            ....:         print("calling init")
            sage: Foo()
            calling classcall
            calling new
            calling init
            <__main__.Foo object at ...>

        This behavior is inherited::

            sage: class Bar(Foo): pass
            sage: Bar()
            calling classcall
            calling new
            calling init
            <__main__.Bar object at ...>

        We now show the usage of ``__classcall_private__``::

            sage: class FooNoInherits(object, metaclass=ClasscallMetaclass):
            ....:     @staticmethod
            ....:     def __classcall_private__(cls):
            ....:         print("calling private classcall")
            ....:         return type.__call__(cls)
            sage: FooNoInherits()
            calling private classcall
            <__main__.FooNoInherits object at ...>

        Here the behavior is not inherited::

            sage: class BarNoInherits(FooNoInherits): pass
            sage: BarNoInherits()
            <__main__.BarNoInherits object at ...>

        We now show the usage of both::

            sage: class Foo2(object, metaclass=ClasscallMetaclass):
            ....:     @staticmethod
            ....:     def __classcall_private__(cls):
            ....:         print("calling private classcall")
            ....:         return type.__call__(cls)
            ....:     @staticmethod
            ....:     def __classcall__(cls):
            ....:         print("calling classcall with %s" % cls)
            ....:         return type.__call__(cls)
            ...
            sage: Foo2()
            calling private classcall
            <__main__.Foo2 object at ...>

            sage: class Bar2(Foo2): pass
            sage: Bar2()
            calling classcall with <class '__main__.Bar2'>
            <__main__.Bar2 object at ...>


        .. rubric:: Discussion

        Typical applications include the implementation of factories or of
        unique representation (see :class:`UniqueRepresentation`). Such
        features are traditionally implemented by either using a wrapper
        function, or fiddling with :meth:`~object.__new__`.

        The benefit, compared with fiddling directly with
        :meth:`~object.__new__` is a clear separation of the three distinct
        roles:

        - ``cls.__classcall__``: what ``cls(<...>)`` does
        - ``cls.__new__``: memory allocation for a *new* instance
        - ``cls.__init__``: initialization of a newly created instance

        The benefit, compared with using a wrapper function, is that the
        user interface has a single handle for the class::

            sage: x = Partition([3,2,2])
            sage: isinstance(x, Partition)          # todo: not implemented

        instead of::

            sage: isinstance(x, sage.combinat.partition.Partition)
            True

        Another difference is that ``__classcall__`` is inherited by
        subclasses, which may be desirable, or not. If not, one should
        instead define the method ``__classcall_private__`` which will
        not be called for subclasses. Specifically, if a class ``cls``
        defines both methods ``__classcall__`` and
        ``__classcall_private__`` then, for any subclass ``sub`` of ``cls``:

        - ``cls(<args>)`` will call ``cls.__classcall_private__(cls, <args>)``
        - ``sub(<args>)`` will call ``cls.__classcall__(sub, <args>)``


        TESTS:

        We check that the idiom ``method_name in cls.__dict__`` works
        for extension types::

           sage: "_sage_" in SageObject.__dict__, "_sage_" in Parent.__dict__
           (True, False)

        We check for memory leaks::

            sage: class NOCALL(object, metaclass=ClasscallMetaclass):
            ....:    pass
            sage: sys.getrefcount(NOCALL())
            1

        We check that exceptions are correctly handled::

            sage: class Exc(object, metaclass=ClasscallMetaclass):
            ....:     @staticmethod
            ....:     def __classcall__(cls):
            ....:         raise ValueError("Calling classcall")
            sage: Exc()
            Traceback (most recent call last):
            ...
            ValueError: Calling classcall
        """
        if cls.classcall is not None:
            return cls.classcall(cls, *args, **kwds)
        else:
            # Fast version of type.__call__(cls, *args, **kwds)
            return (<PyTypeObject*>type).tp_call(cls, args, kwds)

    def __get__(cls, instance, owner):
        r"""
        This method implements instance binding behavior for nested classes.

        Suppose that a class ``Outer`` contains a nested class ``cls`` which
        is an instance of this metaclass. For any object ``obj`` of ``cls``,
        this method implements a instance binding behavior for ``obj.cls`` by
        delegating it to ``cls.__classget__(Outer, obj, owner)`` if available.
        Otherwise, ``obj.cls`` results in ``cls``, as usual.

        Similarly, a class binding as in ``Outer.cls`` is delegated
        to ``cls.__classget__(Outer, None, owner)`` if available and
        to ``cls`` if not.

        .. warning:: for technical reasons, ``__classget__`` must be
            defined as a :func:`staticmethod`, even though it receives
            the class itself as its first argument.

        For technical details, and in particular the description of the
        ``owner`` argument, see the Section :python:`Implementing Descriptor
        <reference/datamodel.html#implementing-descriptors>` in the Python
        reference manual.

        EXAMPLES:

        We show how to implement a nested class ``Outer.Inner`` with a
        binding behavior, as if it was a method of ``Outer``: namely,
        for ``obj`` an instance of ``Outer``, calling
        ``obj.Inner(...)`` is equivalent to ``Outer.Inner(obj, ...)``::

            sage: import functools
            sage: from sage.misc.nested_class import NestedClassMetaclass
            sage: from sage.misc.classcall_metaclass import ClasscallMetaclass
            sage: class Outer(metaclass=NestedClassMetaclass):
            ....:     class Inner(metaclass=ClasscallMetaclass):
            ....:         @staticmethod
            ....:         def __classget__(cls, instance, owner):
            ....:             print("calling __classget__(%s, %s, %s)" % (
            ....:                        cls, instance, owner))
            ....:             if instance is None:
            ....:                 return cls
            ....:             return functools.partial(cls, instance)
            ....:         def __init__(self, instance):
            ....:             self.instance = instance
            sage: obj = Outer()
            sage: bar = obj.Inner()
            calling __classget__(<class '__main__.Outer.Inner'>, <__main__.Outer object at 0x...>, <class '__main__.Outer'>)
            sage: bar.instance == obj
            True

        Calling ``Outer.Inner`` returns the (unbinded) class as usual::

            sage: Inner = Outer.Inner
            calling __classget__(<class '__main__.Outer.Inner'>, None, <class '__main__.Outer'>)
            sage: Inner
            <class '__main__.Outer.Inner'>
            sage: type(bar) is Inner
            True

        .. warning:: Inner has to be a new style class (i.e. a subclass of object).

        .. warning::

            Calling ``obj.Inner`` no longer returns a class::

                sage: bind = obj.Inner
                calling __classget__(<class '__main__.Outer.Inner'>, <__main__.Outer object at 0x...>, <class '__main__.Outer'>)
                sage: bind
                functools.partial(<class '__main__.Outer.Inner'>, <__main__.Outer object at 0x...>)
        """
        if cls.classget:
            return cls.classget(cls, instance, owner)
        else:
            return cls

    def __contains__(cls, x):
        r"""
        This method implements membership testing for a class

        Let ``cls`` be a class in :class:`ClasscallMetaclass`, and consider
        a call of the form::

            x in cls

        If ``cls`` defines a method ``__classcontains__``, then this
        results in a call to::

           cls.__classcontains__(cls, x)

        .. warning:: for technical reasons, ``__classcontains__`` must
            be defined as a :func:`staticmethod`, even though it
            receives the class itself as its first argument.

        EXAMPLES:

        We construct a class which implements membership testing, and
        which contains ``1`` and no other x::

            sage: from sage.misc.classcall_metaclass import ClasscallMetaclass
            sage: class Foo(metaclass=ClasscallMetaclass):
            ....:     @staticmethod
            ....:     def __classcontains__(cls, x):
            ....:         return x == 1
            sage: 1 in Foo
            True
            sage: 2 in Foo
            False

        We now check that for a class without ``__classcontains__``
        method, we emulate the usual error message::

            sage: from sage.misc.classcall_metaclass import ClasscallMetaclass
            sage: class Bar(metaclass=ClasscallMetaclass): pass
            sage: 1 in Bar
            Traceback (most recent call last):
            ...
            TypeError: argument of type 'type' is not iterable
        """
        if cls.classcontains:
            return cls.classcontains(cls, x)
        else:
            return x in object


def typecall(pytype cls, *args, **kwds):
    r"""
    Object construction

    This is a faster equivalent to ``type.__call__(cls, <some arguments>)``.

    INPUT:

    - ``cls`` -- the class used for constructing the instance. It must be
      a builtin type or a new style class (inheriting from :class:`object`).

    EXAMPLES::

        sage: from sage.misc.classcall_metaclass import typecall
        sage: class Foo(object): pass
        sage: typecall(Foo)
        <__main__.Foo object at 0x...>
        sage: typecall(list)
        []
        sage: typecall(Integer, 2)
        2
    """
    return (<PyTypeObject*>type).tp_call(cls, args, kwds)


# Class for timing::

class CRef(object):
    def __init__(self, i):
        """
        TESTS::

            sage: from sage.misc.classcall_metaclass import CRef
            sage: P = CRef(2); P.i
            3
        """
        self.i = i+1


class C2(object, metaclass=ClasscallMetaclass):
    def __init__(self, i):
        """
        TESTS::

            sage: from sage.misc.classcall_metaclass import C2
            sage: P = C2(2); P.i
            3
        """
        self.i = i+1


class C3(object, metaclass = ClasscallMetaclass):
    def __init__(self, i):
        """
        TESTS::

            sage: from sage.misc.classcall_metaclass import C3
            sage: P = C3(2); P.i
            3
        """
        self.i = i+1


class C2C(object, metaclass=ClasscallMetaclass):
    @staticmethod
    def __classcall__(cls, i):
        """
        TESTS::

            sage: from sage.misc.classcall_metaclass import C2C
            sage: C2C(2)
            3
        """
        return i+1


def timeCall(T, int n, *args):
    r"""
    We illustrate some timing when using the classcall mechanism.

    EXAMPLES::

        sage: from sage.misc.classcall_metaclass import (
        ....:     ClasscallMetaclass, CRef, C2, C3, C2C, timeCall)
        sage: timeCall(object, 1000)

    For reference let construct basic objects and a basic Python class::

        sage: %timeit timeCall(object, 1000)   # not tested
        625 loops, best of 3: 41.4 µs per loop

        sage: i1 = int(1); i3 = int(3) # don't use Sage's Integer
        sage: class PRef(object):
        ....:     def __init__(self, i):
        ....:         self.i = i+i1

    For a Python class, compared to the reference class there is a 10%
    overhead in using :class:`ClasscallMetaclass` if there is no classcall
    defined::

        sage: class P(metaclass=ClasscallMetaclass):
        ....:     def __init__(self, i):
        ....:         self.i = i+i1

        sage: %timeit timeCall(PRef, 1000, i3)   # not tested
        625 loops, best of 3: 420 µs per loop
        sage: %timeit timeCall(P, 1000, i3)      # not tested
        625 loops, best of 3: 458 µs per loop

    For a Cython class (not cdef since they doesn't allows metaclasses), the
    overhead is a little larger::

        sage: %timeit timeCall(CRef, 1000, i3)   # not tested
        625 loops, best of 3: 266 µs per loop
        sage: %timeit timeCall(C2, 1000, i3)     # not tested
        625 loops, best of 3: 298 µs per loop

    Let's now compare when there is a classcall defined::

        sage: class PC(object, metaclass=ClasscallMetaclass):
        ....:     @staticmethod
        ....:     def __classcall__(cls, i):
        ....:         return i+i1
        sage: %timeit timeCall(C2C, 1000, i3)   # not tested
        625 loops, best of 3: 148 µs per loop
        sage: %timeit timeCall(PC, 1000, i3)    # not tested
        625 loops, best of 3: 289 µs per loop

    The overhead of the indirection ( ``C(...) ->
    ClasscallMetaclass.__call__(...) -> C.__classcall__(...)``) is
    unfortunately quite large in this case (two method calls instead of
    one). In reasonable usecases, the overhead should be mostly hidden by the
    computations inside the classcall::

        sage: %timeit timeCall(C2C.__classcall__, 1000, C2C, i3)  # not tested
        625 loops, best of 3: 33 µs per loop
        sage: %timeit timeCall(PC.__classcall__, 1000, PC, i3)    # not tested
        625 loops, best of 3: 131 µs per loop

    Finally, there is no significant difference between Cython's V2 and V3
    syntax for metaclass::

        sage: %timeit timeCall(C2, 1000, i3)   # not tested
        625 loops, best of 3: 330 µs per loop
        sage: %timeit timeCall(C3, 1000, i3)   # not tested
        625 loops, best of 3: 328 µs per loop
    """
    cdef int i
    for 0<=i<n:
        T(*args)



r"""
Join features
"""

from . import Feature, FeatureTestResult


class JoinFeature(Feature):
    r"""
    Join of several :class:`~sage.features.Feature` instances.

    EXAMPLES::

        sage: from sage.features import Executable
        sage: from sage.features.join_feature import JoinFeature
        sage: F = JoinFeature("shell-boolean",
        ....:                 (Executable('shell-true', 'true'),
        ....:                  Executable('shell-false', 'false')))
        sage: F.is_present()
        FeatureTestResult('shell-boolean', True)
        sage: F = JoinFeature("asdfghjkl",
        ....:                 (Executable('shell-true', 'true'),
        ....:                  Executable('xxyyyy', 'xxyyyy-does-not-exist')))
        sage: F.is_present()
        FeatureTestResult('xxyyyy', False)
    """
    def __init__(self, name, features, spkg=None, url=None, description=None):
        """
        TESTS:

        The empty join feature is present::

            sage: from sage.features.join_feature import JoinFeature
            sage: JoinFeature("empty", ()).is_present()
            FeatureTestResult('empty', True)
        """
        if spkg is None:
            spkgs = set(f.spkg for f in features if f.spkg)
            if len(spkgs) > 1:
                raise ValueError('given features have more than one spkg; provide spkg argument')
            elif len(spkgs) == 1:
                spkg = next(iter(spkgs))
        if url is None:
            urls = set(f.url for f in features if f.url)
            if len(urls) > 1:
                raise ValueError('given features have more than one url; provide url argument')
            elif len(urls) == 1:
                url = next(iter(urls))
        super().__init__(name, spkg=spkg, url=url, description=description)
        self._features = features

    def _is_present(self):
        r"""
        Test for the presence of the join feature.

        EXAMPLES::

            sage: from sage.features.latte import Latte
            sage: Latte()._is_present()  # optional - latte_int
            FeatureTestResult('latte_int', True)
        """
        for f in self._features:
            test = f._is_present()
            if not test:
                return test
        return FeatureTestResult(self, True)

    def is_functional(self):
        r"""
        Test whether the join feature is functional.

        EXAMPLES::

            sage: from sage.features.latte import Latte
            sage: Latte().is_functional()  # optional - latte_int
            FeatureTestResult('latte_int', True)
        """
        for f in self._features:
            test = f.is_functional()
            if not test:
                return test
        return FeatureTestResult(self, True)



"Exceptions raised by the coercion model"

###############################################################################
#   Sage: Open Source Mathematical Software
#       Copyright (C) 2009 Robert Bradshaw <robertwb@math.washington.edu>
#  Distributed under the terms of the GNU General Public License (GPL)
#  The full text of the GPL is available at:
#                  https://www.gnu.org/licenses/
###############################################################################


class CoercionException(TypeError):
    """
    This is the baseclass of exceptions that the coercion model raises
    when trying to discover coercions. We do not use standard Python
    exceptions to avoid inadvertently catching and suppressing real errors.

    Usually one raises this to indicate the attempted action is not
    implemented/appropriate, but if there are other things to try not
    to immediately abort to the user.
    """
    pass



"""
Lazy format strings
"""


class LazyFormat(str):
    """
    Lazy format strings

    .. NOTE::

        We recommend to use :func:`sage.misc.lazy_string.lazy_string` instead,
        which is both faster and more flexible.

    An instance of :class:`LazyFormat` behaves like a usual format
    string, except that the evaluation of the ``__repr__`` method of
    the formatted arguments it postponed until actual printing.

    EXAMPLES:

    Under normal circumstances, :class:`Lazyformat` strings behave as usual::

        sage: from sage.misc.lazy_format import LazyFormat
        sage: LazyFormat("Got `%s`; expected a list")%3
        Got `3`; expected a list
        sage: LazyFormat("Got `%s`; expected %s")%(3, 2/3)
        Got `3`; expected 2/3

    To demonstrate the lazyness, let us build an object with a broken
    ``__repr__`` method::

        sage: class IDontLikeBeingPrinted(object):
        ....:     def __repr__(self):
        ....:         raise ValueError("Don't ever try to print me !")

    There is no error when binding a lazy format with the broken object::

        sage: lf = LazyFormat("<%s>")%IDontLikeBeingPrinted()

    The error only occurs upon printing::

        sage: lf
        <repr(<sage.misc.lazy_format.LazyFormat at 0x...>) failed: ValueError: Don't ever try to print me !>

    .. rubric:: Common use case:

    Most of the time, ``__repr__`` methods are only called during user
    interaction, and therefore need not be fast; and indeed there are
    objects ``x`` in Sage such ``x.__repr__()`` is time consuming.

    There are however some uses cases where many format strings are
    constructed but not actually printed. This includes error handling
    messages in :mod:`unittest` or :class:`TestSuite` executions::

        sage: QQ._tester().assertTrue(0 in QQ,
        ....:                "%s doesn't contain 0"%QQ)

    In the above ``QQ.__repr__()`` has been called, and the result
    immediately discarded. To demonstrate this we replace ``QQ`` in
    the format string argument with our broken object::

        sage: QQ._tester().assertTrue(True,
        ....:                "%s doesn't contain 0"%IDontLikeBeingPrinted())
        Traceback (most recent call last):
        ...
        ValueError: Don't ever try to print me !

    This behavior can induce major performance penalties when testing.
    Note that this issue does not impact the usual assert::

        sage: assert True, "%s is wrong"%IDontLikeBeingPrinted()

    We now check that :class:`LazyFormat` indeed solves the assertion problem::

        sage: QQ._tester().assertTrue(True,
        ....:               LazyFormat("%s is wrong")%IDontLikeBeingPrinted())
        sage: QQ._tester().assertTrue(False,
        ....:               LazyFormat("%s is wrong")%IDontLikeBeingPrinted())
        Traceback (most recent call last):
        ...
        AssertionError: <unprintable AssertionError object>
    """

    def __mod__(self, args):
        """
        Binds the lazy format string with its parameters

        EXAMPLES::

            sage: from sage.misc.lazy_format import LazyFormat
            sage: form = LazyFormat("<%s>")
            sage: form
            unbound LazyFormat("<%s>")
            sage: form%"params"
            <params>
        """
        if hasattr(self, "_args"): # self is already bound...
            self = LazyFormat(""+self)
        self._args = args
        return self

    def __repr__(self):
        """
        TESTS::

            sage: from sage.misc.lazy_format import LazyFormat
            sage: form = LazyFormat("<%s>")
            sage: form
            unbound LazyFormat("<%s>")
            sage: print(form)
            unbound LazyFormat("<%s>")
            sage: form%"toto"
            <toto>
            sage: print(form % "toto")
            <toto>
            sage: print(str(form % "toto"))
            <toto>
            sage: print((form % "toto").__repr__())
            <toto>
        """
        try:
            args = self._args
        except AttributeError:
            return "unbound LazyFormat(\""+self+"\")"
        else:
            return str.__mod__(self, args)

    __str__ = __repr__



r"""
Homsets

The class :class:`Hom` is the base class used to represent sets of morphisms
between objects of a given category.
:class:`Hom` objects are usually "weakly" cached upon creation so that they
don't have to be generated over and over but can be garbage collected together
with the corresponding objects when these are not strongly ref'ed anymore.

EXAMPLES:

In the following, the :class:`Hom` object is indeed cached::

    sage: K = GF(17)
    sage: H = Hom(ZZ, K)
    sage: H
    Set of Homomorphisms from Integer Ring to Finite Field of size 17
    sage: H is Hom(ZZ, K)
    True

Nonetheless, garbage collection occurs when the original references are
overwritten::

    sage: for p in prime_range(200):
    ....:     K = GF(p)
    ....:     H = Hom(ZZ, K)
    sage: import gc
    sage: _ = gc.collect()
    sage: from sage.rings.finite_rings.finite_field_prime_modn import FiniteField_prime_modn as FF
    sage: L = [x for x in gc.get_objects() if isinstance(x, FF)]
    sage: len(L)
    1
    sage: L
    [Finite Field of size 199]

AUTHORS:

- David Kohel and William Stein

- David Joyner (2005-12-17): added examples

- William Stein (2006-01-14): Changed from Homspace to Homset.

- Nicolas M. Thiery (2008-12-): Updated for the new category framework

- Simon King (2011-12): Use a weak cache for homsets

- Simon King (2013-02): added examples
"""
# ****************************************************************************
#  Copyright (C) 2005 David Kohel <kohel@maths.usyd.edu>, William Stein <wstein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty
#    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
#  See the GNU General Public License for more details; the full text
#  is available at:
#
#                  https://www.gnu.org/licenses/
# ****************************************************************************


from sage.categories.category import Category, JoinCategory
from . import morphism
from sage.structure.parent import Parent, Set_generic
from sage.misc.fast_methods import WithEqualityById
from sage.structure.dynamic_class import dynamic_class
from sage.structure.unique_representation import UniqueRepresentation
from sage.misc.lazy_attribute import lazy_attribute

###################################
# Use the weak "triple" dictionary
# introduced in trac ticket #715
# with weak values, as introduced in
# trac ticket #14159

from sage.structure.coerce_dict import TripleDict
_cache = TripleDict(weak_values=True)

def Hom(X, Y, category=None, check=True):
    """
    Create the space of homomorphisms from X to Y in the category ``category``.

    INPUT:


    - ``X`` -- an object of a category

    - ``Y`` -- an object of a category

    - ``category`` -- a category in which the morphisms must be.
      (default: the meet of the categories of ``X`` and ``Y``)
      Both ``X`` and ``Y`` must belong to that category.

    - ``check`` -- a boolean (default: ``True``): whether to check the
      input, and in particular that ``X`` and ``Y`` belong to
      ``category``.

    OUTPUT: a homset in category

    EXAMPLES::

        sage: V = VectorSpace(QQ,3)
        sage: Hom(V, V)
        Set of Morphisms (Linear Transformations) from
        Vector space of dimension 3 over Rational Field to
        Vector space of dimension 3 over Rational Field
        sage: G = AlternatingGroup(3)
        sage: Hom(G, G)
        Set of Morphisms from Alternating group of order 3!/2 as a permutation group to Alternating group of order 3!/2 as a permutation group in Category of finite enumerated permutation groups
        sage: Hom(ZZ, QQ, Sets())
        Set of Morphisms from Integer Ring to Rational Field in Category of sets

        sage: Hom(FreeModule(ZZ,1), FreeModule(QQ,1))
        Set of Morphisms from Ambient free module of rank 1 over the principal ideal domain Integer Ring to Vector space of dimension 1 over Rational Field in Category of commutative additive groups
        sage: Hom(FreeModule(QQ,1), FreeModule(ZZ,1))
        Set of Morphisms from Vector space of dimension 1 over Rational Field to Ambient free module of rank 1 over the principal ideal domain Integer Ring in Category of commutative additive groups

    Here, we test against a memory leak that has been fixed at :trac:`11521` by
    using a weak cache::

        sage: for p in prime_range(10^3):
        ....:  K = GF(p)
        ....:  a = K(0)
        sage: import gc
        sage: gc.collect()       # random
        624
        sage: from sage.rings.finite_rings.finite_field_prime_modn import FiniteField_prime_modn as FF
        sage: L = [x for x in gc.get_objects() if isinstance(x, FF)]
        sage: len(L), L[0]
        (1, Finite Field of size 997)

    To illustrate the choice of the category, we consider the
    following parents as running examples::

        sage: X = ZZ; X
        Integer Ring
        sage: Y = SymmetricGroup(3); Y
        Symmetric group of order 3! as a permutation group

    By default, the smallest category containing both ``X`` and ``Y``,
    is used::

        sage: Hom(X, Y)
        Set of Morphisms from Integer Ring
         to Symmetric group of order 3! as a permutation group
         in Category of enumerated monoids

    Otherwise, if ``category`` is specified, then ``category`` is used,
    after checking that ``X`` and ``Y`` are indeed in ``category``::

        sage: Hom(X, Y, Magmas())
        Set of Morphisms from Integer Ring to Symmetric group of order 3! as a permutation group in Category of magmas

        sage: Hom(X, Y, Groups())
        Traceback (most recent call last):
        ...
        ValueError: Integer Ring is not in Category of groups

    A parent (or a parent class of a category) may specify how to
    construct certain homsets by implementing a method ``_Hom_(self,
    codomain, category)``. This method should either construct the
    requested homset or raise a ``TypeError``. This hook is currently
    mostly used to create homsets in some specific subclass of
    :class:`Homset` (e.g. :class:`sage.rings.homset.RingHomset`)::

        sage: Hom(QQ,QQ).__class__
        <class 'sage.rings.homset.RingHomset_generic_with_category'>

    Do not call this hook directly to create homsets, as it does not
    handle unique representation::

        sage: Hom(QQ,QQ) == QQ._Hom_(QQ, category=QQ.category())
        True
        sage: Hom(QQ,QQ) is QQ._Hom_(QQ, category=QQ.category())
        False

    TESTS:

    Homset are unique parents::

        sage: k = GF(5)
        sage: H1 = Hom(k,k)
        sage: H2 = Hom(k,k)
        sage: H1 is H2
        True

    Moreover, if no category is provided, then the result is identical
    with the result for the meet of the categories of the domain and
    the codomain::

        sage: Hom(QQ, ZZ) is Hom(QQ,ZZ, Category.meet([QQ.category(), ZZ.category()]))
        True

    Some doc tests in :mod:`sage.rings` (need to) break the unique
    parent assumption. But if domain or codomain are not unique
    parents, then the homset will not fit. That is to say, the hom set
    found in the cache will have a (co)domain that is equal to, but
    not identical with, the given (co)domain.

    By :trac:`9138`, we abandon the uniqueness of homsets, if the
    domain or codomain break uniqueness::

        sage: from sage.rings.polynomial.multi_polynomial_ring import MPolynomialRing_polydict_domain
        sage: P.<x,y,z>=MPolynomialRing_polydict_domain(QQ, 3, order='degrevlex')
        sage: Q.<x,y,z>=MPolynomialRing_polydict_domain(QQ, 3, order='degrevlex')
        sage: P == Q
        True
        sage: P is Q
        False

    Hence, ``P`` and ``Q`` are not unique parents. By consequence, the
    following homsets aren't either::

        sage: H1 = Hom(QQ,P)
        sage: H2 = Hom(QQ,Q)
        sage: H1 == H2
        True
        sage: H1 is H2
        False

    It is always the most recently constructed homset that remains in
    the cache::

        sage: H2 is Hom(QQ,Q)
        True

    Variation on the theme::

        sage: U1 = FreeModule(ZZ,2)
        sage: U2 = FreeModule(ZZ,2,inner_product_matrix=matrix([[1,0],[0,-1]]))
        sage: U1 == U2, U1 is U2
        (False, False)
        sage: V = ZZ^3
        sage: H1 = Hom(U1, V); H2 = Hom(U2, V)
        sage: H1 == H2, H1 is H2
        (False, False)
        sage: H1 = Hom(V, U1); H2 = Hom(V, U2)
        sage: H1 == H2, H1 is H2
        (False, False)

    Since :trac:`11900`, the meet of the categories of the given arguments is
    used to determine the default category of the homset. This can also be a
    join category, as in the following example::

        sage: PA = Parent(category=Algebras(QQ))
        sage: PJ = Parent(category=Rings() & Modules(QQ))
        sage: Hom(PA,PJ)
        Set of Homomorphisms from <sage.structure.parent.Parent object at ...> to <sage.structure.parent.Parent object at ...>
        sage: Hom(PA,PJ).category()
        Category of homsets of unital magmas and right modules over Rational Field and left modules over Rational Field
        sage: Hom(PA,PJ, Rngs())
        Set of Morphisms from <sage.structure.parent.Parent object at ...> to <sage.structure.parent.Parent object at ...> in Category of rngs

    .. TODO::

        - Design decision: how much of the homset comes from the
          category of ``X`` and ``Y``, and how much from the specific
          ``X`` and ``Y``.  In particular, do we need several parent
          classes depending on ``X`` and ``Y``, or does the difference
          only lie in the elements (i.e.  the morphism), and of course
          how the parent calls their constructors.
        - Specify the protocol for the ``_Hom_`` hook in case of ambiguity
          (e.g. if both a parent and some category thereof provide one).

    TESTS:

    Facade parents over plain Python types are supported::

        sage: from sage.sets.pythonclass import Set_PythonType
        sage: R = Set_PythonType(int)
        sage: S = Set_PythonType(float)
        sage: Hom(R, S)
        Set of Morphisms from Set of Python objects of class 'int' to Set of Python objects of class 'float' in Category of sets

    Checks that the domain and codomain are in the specified
    category. Case of a non parent::

        sage: S = SimplicialComplex([[1,2], [1,4]]); S.rename("S")
        sage: Hom(S, S, SimplicialComplexes())
        Set of Morphisms from S to S in Category of finite simplicial complexes

        sage: Hom(Set(), S, Sets())
        Set of Morphisms from {} to S in Category of sets

        sage: Hom(S, Set(), Sets())
        Set of Morphisms from S to {} in Category of sets

        sage: H = Hom(S, S, ChainComplexes(QQ))
        Traceback (most recent call last):
        ...
        ValueError: S is not in Category of chain complexes over Rational Field

    Those checks are done with the natural idiom ``X in category``,
    and not ``X.category().is_subcategory(category)`` as it used to be
    before :trac:`16275` (see :trac:`15801` for a real use case)::

        sage: class PermissiveCategory(Category):
        ....:     def super_categories(self): return [Objects()]
        ....:     def __contains__(self, X): return True
        sage: C = PermissiveCategory(); C.rename("Permissive category")
        sage: S.category().is_subcategory(C)
        False
        sage: S in C
        True
        sage: Hom(S, S, C)
        Set of Morphisms from S to S in Permissive category

    With ``check=False``, uninitialized parents, as can appear upon
    unpickling, are supported. Case of a parent::

        sage: cls = type(Set())
        sage: S = unpickle_newobj(cls, ())  # A non parent
        sage: H = Hom(S, S, SimplicialComplexes(), check=False)
        sage: H = Hom(S, S, Sets(),                check=False)
        sage: H = Hom(S, S, ChainComplexes(QQ),    check=False)

    Case of a non parent::

        sage: cls = type(SimplicialComplex([[1,2], [1,4]]))
        sage: S = unpickle_newobj(cls, ())
        sage: H = Hom(S, S, Sets(),                check=False)
        sage: H = Hom(S, S, Groups(),              check=False)
        sage: H = Hom(S, S, SimplicialComplexes(), check=False)

    Typical example where unpickling involves calling Hom on an
    uninitialized parent::

        sage: P.<x,y> = QQ['x,y']
        sage: Q = P.quotient([x^2-1,y^2-1])
        sage: q = Q.an_element()
        sage: explain_pickle(dumps(Q))
        pg_...
        ... = pg_dynamic_class('QuotientRing_generic_with_category', (pg_QuotientRing_generic, pg_getattr(..., 'parent_class')), None, None, pg_QuotientRing_generic)
        si... = unpickle_newobj(..., ())
        ...
        si... = pg_unpickle_MPolynomialRing_libsingular(..., ('x', 'y'), ...)
        si... = ... pg_Hom(si..., si..., ...) ...
        sage: Q == loads(dumps(Q))
        True

    Check that the ``_Hom_`` method of the ``category`` input is used::

        sage: from sage.categories.category_types import Category_over_base_ring
        sage: class ModulesWithHom(Category_over_base_ring):
        ....:     def super_categories(self):
        ....:         return [Modules(self.base_ring())]
        ....:     class ParentMethods:
        ....:         def _Hom_(self, Y, category=None):
        ....:             print("Modules")
        ....:             raise TypeError
        sage: class AlgebrasWithHom(Category_over_base_ring):
        ....:     def super_categories(self):
        ....:         return [Algebras(self.base_ring()), ModulesWithHom(self.base_ring())]
        ....:     class ParentMethods:
        ....:         def _Hom_(self, Y, category=None):
        ....:             R = self.base_ring()
        ....:             if category is not None and category.is_subcategory(Algebras(R)):
        ....:                 print("Algebras")
        ....:             raise TypeError
        sage: from sage.structure.element import Element
        sage: class Foo(Parent):
        ....:     def _coerce_map_from_base_ring(self):
        ....:         return self._generic_coerce_map(self.base_ring())
        ....:     class Element(Element):
        ....:         pass
        sage: X = Foo(base=QQ, category=AlgebrasWithHom(QQ))
        sage: H = Hom(X, X, ModulesWithHom(QQ))
        Modules
    """
    # This should use cache_function instead
    # However some special handling is currently needed for
    # domains/docomains that break the unique parent condition. Also,
    # at some point, it somehow broke the coercion (see e.g. sage -t
    # sage.rings.real_mpfr). To be investigated.
    global _cache
    key = (X,Y,category)
    try:
        H = _cache[key]
    except KeyError:
        H = None
    if H is not None:
        # Return H unless the domain or codomain breaks the unique parent condition
        if H.domain() is X and H.codomain() is Y:
            return H

    # Determines the category
    if category is None:
        category = X.category()._meet_(Y.category())
        # Recurse to make sure that Hom(X, Y) and Hom(X, Y, category) are identical
        # No need to check the input again
        H = Hom(X, Y, category, check=False)
    else:
        if check:
            if not isinstance(category, Category):
                raise TypeError("Argument category (= {}) must be a category.".format(category))
            for O in [X, Y]:
                try:
                    category_mismatch = O not in category
                except Exception:
                    # An error should not happen, this here is just to be on
                    # the safe side.
                    category_mismatch = True
                # A category mismatch does not necessarily mean that an error
                # should be raised. Instead, it could be the case that we are
                # unpickling an old pickle (that doesn't set the "check"
                # argument to False). In this case, it could be that the
                # (co)domain is not properly initialised, which we are
                # checking now. See trac #16275 and #14793.
                if category_mismatch and O._is_category_initialized():
                    # At this point, we can be rather sure that O is properly
                    # initialised, and thus its string representation is
                    # available for the following error message. It simply
                    # belongs to the wrong category.
                    raise ValueError("{} is not in {}".format(O, category))

        # Construct H
        try: # _Hom_ hook from the parent
            H = X._Hom_(Y, category)
        except (AttributeError, TypeError):
            # Workaround in case the above fails, but the category
            # also provides a _Hom_ hook.
            # FIXME:
            # - If X._Hom_ actually comes from category and fails, it
            #   will be called twice.
            # - This is bound to fail if X is an extension type and
            #   does not actually inherit from category.parent_class
            # For join categories, we check all of the direct super
            #   categories as the parent_class of the join category is
            #   not (necessarily) inherited and join categories do not
            #   implement a _Hom_ (see trac #23418).
            if not isinstance(category, JoinCategory):
                cats = [category]
            else:
                cats = category.super_categories()
            H = None
            for C in cats:
                try:
                    H = C.parent_class._Hom_(X, Y, category=category)
                    break
                except (AttributeError, TypeError):
                    pass
            if H is None:
                # By default, construct a plain homset.
                H = Homset(X, Y, category=category, check=check)
    _cache[key] = H
    if isinstance(X, UniqueRepresentation) and isinstance(Y, UniqueRepresentation):
        if not isinstance(H, WithEqualityById):
            try:
                H.__class__ = dynamic_class(H.__class__.__name__+"_with_equality_by_id", (WithEqualityById, H.__class__), doccls=H.__class__)
            except Exception:
                pass
    return H

def hom(X, Y, f):
    """
    Return ``Hom(X,Y)(f)``, where ``f`` is data that defines an element of
    ``Hom(X,Y)``.

    EXAMPLES::

        sage: phi = hom(QQ['x'], QQ, [2])
        sage: phi(x^2 + 3)
        7
    """
    return Hom(X,Y)(f)

def End(X, category=None):
    r"""
    Create the set of endomorphisms of ``X`` in the category category.

    INPUT:

    -  ``X`` -- anything

    -  ``category`` -- (optional) category in which to coerce ``X``

    OUTPUT:

    A set of endomorphisms in category

    EXAMPLES::

        sage: V = VectorSpace(QQ, 3)
        sage: End(V)
        Set of Morphisms (Linear Transformations) from
        Vector space of dimension 3 over Rational Field to
        Vector space of dimension 3 over Rational Field

    ::

        sage: G = AlternatingGroup(3)
        sage: S = End(G); S
        Set of Morphisms from Alternating group of order 3!/2 as a permutation group to Alternating group of order 3!/2 as a permutation group in Category of finite enumerated permutation groups
        sage: from sage.categories.homset import is_Endset
        sage: is_Endset(S)
        True
        sage: S.domain()
        Alternating group of order 3!/2 as a permutation group

    To avoid creating superfluous categories, a homset in a category
    ``Cs()`` is in the homset category of the lowest full super category
    ``Bs()`` of ``Cs()`` that implements ``Bs.Homsets`` (or the join
    thereof if there are several). For example, finite groups form a
    full subcategory of unital magmas: any unital magma morphism
    between two finite groups is a finite group morphism. Since finite
    groups currently implement nothing more than unital magmas about
    their homsets, we have::

        sage: G = GL(3,3)
        sage: G.category()
        Category of finite groups
        sage: H = Hom(G,G)
        sage: H.homset_category()
        Category of finite groups
        sage: H.category()
        Category of endsets of unital magmas

    Similarly, a ring morphism just needs to preserve addition,
    multiplication, zero, and one. Accordingly, and since the category
    of rings implements nothing specific about its homsets, a ring
    homset is currently constructed in the category of homsets of
    unital magmas and unital additive magmas::

        sage: H = Hom(ZZ,ZZ,Rings())
        sage: H.category()
        Category of endsets of unital magmas and additive unital additive magmas
    """
    return Hom(X,X, category)

def end(X, f):
    """
    Return ``End(X)(f)``, where ``f`` is data that defines an element of
    ``End(X)``.

    EXAMPLES::

        sage: R.<x> = QQ[]
        sage: phi = end(R, [x + 1])
        sage: phi
        Ring endomorphism of Univariate Polynomial Ring in x over Rational Field
          Defn: x |--> x + 1
        sage: phi(x^2 + 5)
        x^2 + 2*x + 6
    """
    return End(X)(f)


class Homset(Set_generic):
    """
    The class for collections of morphisms in a category.

    EXAMPLES::

        sage: H = Hom(QQ^2, QQ^3)
        sage: loads(H.dumps()) is H
        True

    Homsets of unique parents are unique as well::

        sage: H = End(AffineSpace(2, names='x,y'))
        sage: loads(dumps(AffineSpace(2, names='x,y'))) is AffineSpace(2, names='x,y')
        True
        sage: loads(dumps(H)) is H
        True

    Conversely, homsets of non-unique parents are non-unique::

        sage: H = End(ProductProjectiveSpaces(QQ, [1, 1]))
        sage: loads(dumps(ProductProjectiveSpaces(QQ, [1, 1]))) is ProductProjectiveSpaces(QQ, [1, 1])
        False
        sage: loads(dumps(ProductProjectiveSpaces(QQ, [1, 1]))) == ProductProjectiveSpaces(QQ, [1, 1])
        True
        sage: loads(dumps(H)) is H
        False
        sage: loads(dumps(H)) == H
        True
    """
    def __init__(self, X, Y, category=None, base = None, check=True):
        r"""
        TESTS::

            sage: X = ZZ['x']; X.rename("X")
            sage: Y = ZZ['y']; Y.rename("Y")
            sage: f = X.hom([0], Y)
            sage: class MyHomset(Homset):
            ....:     def _an_element_(self):
            ....:         return sage.categories.morphism.SetMorphism(self, f)
            sage: import __main__; __main__.MyHomset = MyHomset # fakes MyHomset being defined in a Python module
            sage: H = MyHomset(X, Y, category=Monoids(), base = ZZ)
            sage: H
            Set of Morphisms from X to Y in Category of monoids
            sage: TestSuite(H).run()

            sage: H = MyHomset(X, Y, category=1, base = ZZ)
            Traceback (most recent call last):
            ...
            TypeError: category (=1) must be a category

            sage: H = MyHomset(X, Y, category=1, base = ZZ, check = False)
            Traceback (most recent call last):
            ...
            AttributeError: 'sage.rings.integer.Integer' object has no attribute 'Homsets'
            sage: P.<t> = ZZ[]
            sage: f = P.hom([1/2*t])
            sage: f.parent().domain()
            Univariate Polynomial Ring in t over Integer Ring
            sage: f.domain() is f.parent().domain()
            True

        Test that ``base_ring`` is initialized properly::

            sage: R = QQ['x']
            sage: Hom(R, R).base_ring()
            Rational Field
            sage: Hom(R, R, category=Sets()).base_ring()
            sage: Hom(R, R, category=Modules(QQ)).base_ring()
            Rational Field
            sage: Hom(QQ^3, QQ^3, category=Modules(QQ)).base_ring()
            Rational Field

        For whatever it's worth, the ``base`` arguments takes precedence::

            sage: MyHomset(ZZ^3, ZZ^3, base = QQ).base_ring()
            Rational Field
        """
        self._domain = X
        self._codomain = Y
        if category is None:
            category = X.category()
        self.__category = category
        if check:
            if not isinstance(category, Category):
                raise TypeError("category (=%s) must be a category"%category)
            #if not X in category:
            #    raise TypeError, "X (=%s) must be in category (=%s)"%(X, category)
            #if not Y in category:
            #    raise TypeError, "Y (=%s) must be in category (=%s)"%(Y, category)

        if base is None and hasattr(category, "WithBasis"):
            # The above is a lame but fast check that category is a
            # subcategory of Modules(...). That will do until
            # CategoryObject.base_ring will be gone and not prevent
            # anymore from implementing base_ring in Modules.Homsets.ParentMethods.
            # See also #15801.
            base = X.base_ring()

        Parent.__init__(self, base = base,
                        category = category.Endsets() if X is Y else category.Homsets())

    def __reduce__(self):
        """
        Implement pickling by construction for Homsets.

        Homsets are unpickled using the function
        :func:`~sage.categories.homset.Hom` which is cached:
        ``Hom(domain, codomain, category, check=False)``.

        .. NOTE::

            It can happen, that ``Hom(X,X)`` is called during
            unpickling with an uninitialized instance ``X`` of a Python
            class. In some of these cases, testing that ``X in
            category`` can trigger ``X.category()``. This in turn can
            raise a error, or return a too large category (``Sets()``,
            for example) and (worse!) assign this larger category to
            the ``X._category`` cdef attribute, so that it would
            subsequently seem that ``X``'s category was initialised.

            Beside speed considerations, this is the main rationale
            for disabling checks upon unpickling.

            .. SEEALSO:: :trac:`14793`, :trac:`16275`

        EXAMPLES::

            sage: H = Hom(QQ^2, QQ^3)
            sage: H.__reduce__()
            (<function Hom at ...>,
             (Vector space of dimension 2 over Rational Field,
              Vector space of dimension 3 over Rational Field,
              Category of finite dimensional vector spaces with basis over
                 (number fields and quotient fields and metric spaces),
              False))

        TESTS::

            sage: loads(H.dumps()) is H
            True

        Homsets of non-unique parents are non-unique as well::

            sage: G = PermutationGroup([[(1,2,3),(4,5)],[(3,4)]])
            sage: G is loads(dumps(G))
            False
            sage: H = Hom(G,G)
            sage: H is loads(dumps(H))
            False
            sage: H == loads(dumps(H))
            True
        """
        return Hom, (self._domain, self._codomain, self.__category, False)

    def _repr_(self):
        """
        TESTS::

            sage: Hom(ZZ^2, QQ, category=Sets())._repr_()
            'Set of Morphisms from Ambient free module of rank 2 over the principal ideal domain Integer Ring to Rational Field in Category of sets'
        """
        return "Set of Morphisms from {} to {} in {}".format(self._domain,
            self._codomain, self.__category)

    def __hash__(self):
        """
        The hash is obtained from domain, codomain and base.

        TESTS::

            sage: hash(Hom(ZZ, QQ)) == hash((ZZ, QQ, ZZ))
            True
            sage: hash(Hom(QQ, ZZ)) == hash((QQ, ZZ, QQ))
            True

            sage: E = EllipticCurve('37a')
            sage: H = E(0).parent(); H
            Abelian group of points on Elliptic Curve defined by y^2 + y = x^3 - x over Rational Field
            sage: hash(H) == hash((H.domain(), H.codomain(), H.base()))
            True
        """
        return hash((self._domain, self._codomain, self.base()))

    def __bool__(self):
        """
        TESTS::

            sage: bool(Hom(ZZ, QQ))
            True
        """
        return True

    __nonzero__ = __bool__

    def homset_category(self):
        """
        Return the category that this is a Hom in, i.e., this is typically
        the category of the domain or codomain object.

        EXAMPLES::

            sage: H = Hom(AlternatingGroup(4), AlternatingGroup(7))
            sage: H.homset_category()
            Category of finite enumerated permutation groups
        """
        return self.__category

    def _element_constructor_(self, x, check=None, **options):
        r"""
        Construct a morphism in this homset from ``x`` if possible.

        EXAMPLES::

            sage: H = Hom(SymmetricGroup(4), SymmetricGroup(7))
            sage: phi = Hom(SymmetricGroup(5), SymmetricGroup(6)).natural_map()
            sage: phi
            Coercion morphism:
              From: Symmetric group of order 5! as a permutation group
              To:   Symmetric group of order 6! as a permutation group

        When converting `\phi` into `H`, some coerce maps are applied. Note
        that (in contrast to what is stated in the following string
        representation) it is safe to use the resulting map, since a composite
        map prevents the codomains of all constituent maps from garbage
        collection, if there is a strong reference to its domain (which is the
        case here)::

            sage: H(phi)
            Composite map:
              From: Symmetric group of order 4! as a permutation group
              To:   Symmetric group of order 7! as a permutation group
              Defn:   (map internal to coercion system -- copy before use)
                    Coercion map:
                      From: Symmetric group of order 4! as a permutation group
                      To:   Symmetric group of order 5! as a permutation group
                    then
                      Coercion morphism:
                      From: Symmetric group of order 5! as a permutation group
                      To:   Symmetric group of order 6! as a permutation group
                    then
                      (map internal to coercion system -- copy before use)
                    Coercion map:
                      From: Symmetric group of order 6! as a permutation group
                      To:   Symmetric group of order 7! as a permutation group

        Also note that making a copy of the resulting map will automatically
        make strengthened copies of the composed maps::

            sage: copy(H(phi))
            Composite map:
              From: Symmetric group of order 4! as a permutation group
              To:   Symmetric group of order 7! as a permutation group
              Defn:   Coercion map:
                      From: Symmetric group of order 4! as a permutation group
                      To:   Symmetric group of order 5! as a permutation group
                    then
                      Coercion morphism:
                      From: Symmetric group of order 5! as a permutation group
                      To:   Symmetric group of order 6! as a permutation group
                    then
                      Coercion map:
                      From: Symmetric group of order 6! as a permutation group
                      To:   Symmetric group of order 7! as a permutation group
            sage: H = Hom(ZZ, ZZ, Sets())
            sage: f = H( lambda x: x + 1 )
            sage: f.parent()
            Set of Morphisms from Integer Ring to Integer Ring in Category of sets
            sage: f.domain()
            Integer Ring
            sage: f.codomain()
            Integer Ring
            sage: f(1), f(2), f(3)
            (2, 3, 4)

            sage: H = Hom(Set([1,2,3]), Set([1,2,3]))
            sage: f = H( lambda x: 4-x )
            sage: f.parent()
            Set of Morphisms from {1, 2, 3} to {1, 2, 3} in Category of finite sets
            sage: f(1), f(2), f(3) # todo: not implemented

            sage: H = Hom(ZZ, QQ, Sets())
            sage: f = H( ConstantFunction(2/3) )
            sage: f.parent()
            Set of Morphisms from Integer Ring to Rational Field in Category of sets
            sage: f(1), f(2), f(3)
            (2/3, 2/3, 2/3)

        By :trac:`14711`, conversion and coerce maps should be copied
        before using them outside of the coercion system::

            sage: H = Hom(ZZ,QQ['t'], CommutativeAdditiveGroups())
            sage: P.<t> = ZZ[]
            sage: f = P.hom([2*t])
            sage: phi = H._generic_convert_map(f.parent()); phi
            Conversion map:
              From: Set of Homomorphisms from Univariate Polynomial Ring in t over Integer Ring to Univariate Polynomial Ring in t over Integer Ring
              To:   Set of Morphisms from Integer Ring to Univariate Polynomial Ring in t over Rational Field in Category of commutative additive groups
            sage: H._generic_convert_map(f.parent())(f)
            Composite map:
              From: Integer Ring
              To:   Univariate Polynomial Ring in t over Rational Field
              Defn:   (map internal to coercion system -- copy before use)
                    Polynomial base injection morphism:
                      From: Integer Ring
                      To:   Univariate Polynomial Ring in t over Integer Ring
                    then
                      Ring endomorphism of Univariate Polynomial Ring in t over Integer Ring
                      Defn: t |--> 2*t
                    then
                      (map internal to coercion system -- copy before use)
                    Ring morphism:
                      From: Univariate Polynomial Ring in t over Integer Ring
                      To:   Univariate Polynomial Ring in t over Rational Field
            sage: copy(H._generic_convert_map(f.parent())(f))
            Composite map:
              From: Integer Ring
              To:   Univariate Polynomial Ring in t over Rational Field
              Defn:   Polynomial base injection morphism:
                      From: Integer Ring
                      To:   Univariate Polynomial Ring in t over Integer Ring
                    then
                      Ring endomorphism of Univariate Polynomial Ring in t over Integer Ring
                      Defn: t |--> 2*t
                    then
                      Ring morphism:
                      From: Univariate Polynomial Ring in t over Integer Ring
                      To:   Univariate Polynomial Ring in t over Rational Field
                      Defn: Induced from base ring by
                            Natural morphism:
                              From: Integer Ring
                              To:   Rational Field

        TESTS::

            sage: G.<x,y,z> = FreeGroup()
            sage: H = Hom(G, G)
            sage: H(H.identity())
            Identity endomorphism of Free Group on generators {x, y, z}
            sage: H()
            Traceback (most recent call last):
            ...
            TypeError: unable to convert 0 to an element of
             Set of Morphisms from Free Group on generators {x, y, z}
             to Free Group on generators {x, y, z} in Category of infinite groups
            sage: H("whatever")
            Traceback (most recent call last):
            ...
            TypeError: unable to convert 'whatever' to an element of
             Set of Morphisms from Free Group on generators {x, y, z}
             to Free Group on generators {x, y, z} in Category of infinite groups
            sage: HH = Hom(H, H)
            sage: HH(HH.identity(), foo="bar")
            Traceback (most recent call last):
            ...
            NotImplementedError: no keywords are implemented for
             constructing elements of ...

        AUTHORS:

        - Robert Bradshaw, with changes by Nicolas M. Thiery
        """
        if options:
            # TODO: this is specific for ModulesWithBasis; generalize
            # this to allow homsets and categories to provide more
            # morphism constructors (on_algebra_generators, ...)
            try:
                call_with_keywords = self.__call_on_basis__
            except AttributeError:
                if 'base_map' in options:
                    raise NotImplementedError("base_map not supported for this Homset; you may need to specify a category")
                raise NotImplementedError("no keywords are implemented for constructing elements of {}".format(self))
            options.setdefault("category", self.homset_category())
            return call_with_keywords(**options)

        if isinstance(x, morphism.Morphism):
            if x.domain() != self.domain():
                mor = x.domain()._internal_coerce_map_from(self.domain())
                if mor is None:
                    raise TypeError("Incompatible domains: x (=%s) cannot be an element of %s"%(x,self))
                x = x * mor
            if x.codomain() != self.codomain():
                mor = self.codomain()._internal_coerce_map_from(x.codomain())
                if mor is None:
                    raise TypeError("Incompatible codomains: x (=%s) cannot be an element of %s"%(x,self))
                x = mor * x
            return x

        if callable(x):
            return self.element_class_set_morphism(self, x)

        raise TypeError("unable to convert {!r} to an element of {}".format(x, self))

    @lazy_attribute
    def _abstract_element_class(self):
        """
        An abstract class for the elements of this homset.

        This class is built from the element class of the homset
        category and the morphism class of the category.  This makes
        it possible for a category to provide code for its morphisms
        and for morphisms of all its subcategories, full or not.

        .. NOTE::

            The element class of ``C.Homsets()`` will be inherited by
            morphisms in *full* subcategories of ``C``, while the morphism
            class of ``C`` will be inherited by *all* subcategories of
            ``C``. Hence, if some feature of a morphism depends on the
            algebraic properties of the homsets, it should be implemented by
            ``C.Homsets.ElementMethods``, but if it depends only on the
            algebraic properties of domain and codomain, it should be
            implemented in ``C.MorphismMethods``.

            At this point, the homset element classes take precedence over the
            morphism classes. But this may be subject to change.


        .. TODO::

            - Make sure this class is shared whenever possible.
            - Flatten join category classes

        .. SEEALSO::

            - :meth:`Parent._abstract_element_class`

        EXAMPLES:

        Let's take a homset of finite commutative groups as example; at
        this point this is the simplest one to create (gosh)::

            sage: cat = Groups().Finite().Commutative()
            sage: C3 = PermutationGroup([(1,2,3)])
            sage: C3._refine_category_(cat)
            sage: C2 = PermutationGroup([(1,2)])
            sage: C2._refine_category_(cat)
            sage: H = Hom(C3, C2, cat)
            sage: H.homset_category()
            Category of finite commutative groups
            sage: H.category()
            Category of homsets of unital magmas
            sage: cls = H._abstract_element_class; cls
            <class 'sage.categories.homsets.GroupHomset_libgap_with_category._abstract_element_class'>
            sage: cls.__bases__ == (H.category().element_class, H.homset_category().morphism_class)
            True

        A morphism of finite commutative semigroups is also a morphism
        of semigroups, of magmas, ...; it thus inherits code from all
        those categories::

            sage: issubclass(cls, Semigroups().Finite().morphism_class)
            True
            sage: issubclass(cls, Semigroups().morphism_class)
            True
            sage: issubclass(cls, Magmas().Commutative().morphism_class)
            True
            sage: issubclass(cls, Magmas().morphism_class)
            True
            sage: issubclass(cls, Sets().morphism_class)
            True

        Recall that FiniteMonoids() is a full subcategory of
        ``Monoids()``, but not of ``FiniteSemigroups()``. Thus::

            sage: issubclass(cls, Monoids().Finite().Homsets().element_class)
            True
            sage: issubclass(cls, Semigroups().Finite().Homsets().element_class)
            False
        """
        class_name = "%s._abstract_element_class"%self.__class__.__name__
        return dynamic_class(class_name, (self.category().element_class, self.homset_category().morphism_class))

    @lazy_attribute
    def element_class_set_morphism(self):
        """
        A base class for elements of this homset which are
        also ``SetMorphism``, i.e. implemented by mean of a
        Python function.

        This is currently plain ``SetMorphism``, without inheritance
        from categories.

        .. TODO::

            Refactor during the upcoming homset cleanup.

        EXAMPLES::

            sage: H = Hom(ZZ, ZZ)
            sage: H.element_class_set_morphism
            <class 'sage.categories.morphism.SetMorphism'>
        """
        return self.__make_element_class__(morphism.SetMorphism)

    def __eq__(self, other):
        """
        For two homsets, it is tested whether the domain, the codomain and
        the category coincide.

        EXAMPLES::

            sage: H1 = Hom(ZZ,QQ, CommutativeAdditiveGroups())
            sage: H2 = Hom(ZZ,QQ)
            sage: H1 == H2
            False
            sage: H1 == loads(dumps(H1))
            True
        """
        if not isinstance(other, Homset):
            return False
        return (self._domain == other._domain
                and self._codomain == other._codomain
                and self.__category == other.__category)

    def __ne__(self, other):
        """
        Check for not-equality of ``self`` and ``other``.

        EXAMPLES::

            sage: H1 = Hom(ZZ,QQ, CommutativeAdditiveGroups())
            sage: H2 = Hom(ZZ,QQ)
            sage: H3 = Hom(ZZ['t'],QQ, CommutativeAdditiveGroups())
            sage: H4 = Hom(ZZ,QQ['t'], CommutativeAdditiveGroups())
            sage: H1 != H2
            True
            sage: H1 != loads(dumps(H1))
            False
            sage: H1 != H3 != H4 != H1
            True
        """
        return not (self == other)
    
    def __contains__(self, x):
        """
        Test whether the parent of the argument is ``self``.

        TESTS::

            sage: P.<t> = ZZ[]
            sage: f = P.hom([1/2*t])
            sage: f in Hom(ZZ['t'],QQ['t'])  # indirect doctest
            True
            sage: f in Hom(ZZ['t'],QQ['t'], CommutativeAdditiveGroups())  # indirect doctest
            False
        """
        try:
            return x.parent() == self
        except AttributeError:
            pass
        return False

    def natural_map(self):
        """
        Return the "natural map" of this homset.

        .. NOTE::

            By default, a formal coercion morphism is returned.

        EXAMPLES::

            sage: H = Hom(ZZ['t'],QQ['t'], CommutativeAdditiveGroups())
            sage: H.natural_map()
            Coercion morphism:
              From: Univariate Polynomial Ring in t over Integer Ring
              To:   Univariate Polynomial Ring in t over Rational Field
            sage: H = Hom(QQ['t'],GF(3)['t'])
            sage: H.natural_map()
            Traceback (most recent call last):
            ...
            TypeError: natural coercion morphism from Univariate Polynomial Ring in t over Rational Field to Univariate Polynomial Ring in t over Finite Field of size 3 not defined
        """
        return morphism.FormalCoercionMorphism(self)   # good default in many cases

    def identity(self):
        """
        The identity map of this homset.

        .. NOTE::

            Of course, this only exists for sets of endomorphisms.

        EXAMPLES::

            sage: H = Hom(QQ,QQ)
            sage: H.identity()
            Identity endomorphism of Rational Field
            sage: H = Hom(ZZ,QQ)
            sage: H.identity()
            Traceback (most recent call last):
            ...
            TypeError: Identity map only defined for endomorphisms. Try natural_map() instead.
            sage: H.natural_map()
            Natural morphism:
              From: Integer Ring
              To:   Rational Field
        """
        if self.is_endomorphism_set():
            return morphism.IdentityMorphism(self)
        else:
            raise TypeError("Identity map only defined for endomorphisms. Try natural_map() instead.")

    def one(self):
        """
        The identity map of this homset.

        .. NOTE::

            Of course, this only exists for sets of endomorphisms.

        EXAMPLES::

            sage: K = GaussianIntegers()
            sage: End(K).one()
            Identity endomorphism of Gaussian Integers in Number Field in I with defining polynomial x^2 + 1 with I = 1*I
        """
        return self.identity()

    def domain(self):
        """
        Return the domain of this homset.

        EXAMPLES::

            sage: P.<t> = ZZ[]
            sage: f = P.hom([1/2*t])
            sage: f.parent().domain()
            Univariate Polynomial Ring in t over Integer Ring
            sage: f.domain() is f.parent().domain()
            True
        """
        return self._domain

    def codomain(self):
        """
        Return the codomain of this homset.

        EXAMPLES::

            sage: P.<t> = ZZ[]
            sage: f = P.hom([1/2*t])
            sage: f.parent().codomain()
            Univariate Polynomial Ring in t over Rational Field
            sage: f.codomain() is f.parent().codomain()
            True
        """
        return self._codomain

    def reversed(self):
        """
        Return the corresponding homset, but with the domain and codomain
        reversed.

        EXAMPLES::

            sage: H = Hom(ZZ^2, ZZ^3); H
            Set of Morphisms from Ambient free module of rank 2 over
             the principal ideal domain Integer Ring to Ambient free module
             of rank 3 over the principal ideal domain Integer Ring in
             Category of finite dimensional modules with basis over (euclidean
             domains and infinite enumerated sets and metric spaces)
            sage: type(H)
            <class 'sage.modules.free_module_homspace.FreeModuleHomspace_with_category'>
            sage: H.reversed()
            Set of Morphisms from Ambient free module of rank 3 over
             the principal ideal domain Integer Ring to Ambient free module
             of rank 2 over the principal ideal domain Integer Ring in
             Category of finite dimensional modules with basis over (euclidean
             domains and infinite enumerated sets and metric spaces)
            sage: type(H.reversed())
            <class 'sage.modules.free_module_homspace.FreeModuleHomspace_with_category'>
        """
        return Hom(self.codomain(), self.domain(), category = self.homset_category())


# Really needed???
class HomsetWithBase(Homset):
    def __init__(self, X, Y, category=None, check=True, base=None):
        r"""
        TESTS::

            sage: X = ZZ['x']; X.rename("X")
            sage: Y = ZZ['y']; Y.rename("Y")
            sage: f = X.hom([0], Y)
            sage: class MyHomset(HomsetWithBase):
            ....:     def _an_element_(self):
            ....:         return sage.categories.morphism.SetMorphism(self, f)
            sage: import __main__; __main__.MyHomset = MyHomset # fakes MyHomset being defined in a Python module
            sage: H = MyHomset(X, Y, category=Monoids())
            sage: H
            Set of Morphisms from X to Y in Category of monoids
            sage: H.base()
            Integer Ring
            sage: TestSuite(H).run()
        """
        if base is None:
            base = X.base_ring()
        Homset.__init__(self, X, Y, check=check, category=category, base = base)

def is_Homset(x):
    """
    Return ``True`` if ``x`` is a set of homomorphisms in a category.

    EXAMPLES::

        sage: from sage.categories.homset import is_Homset
        sage: P.<t> = ZZ[]
        sage: f = P.hom([1/2*t])
        sage: is_Homset(f)
        False
        sage: is_Homset(f.category())
        False
        sage: is_Homset(f.parent())
        True
    """
    return isinstance(x, Homset)

def is_Endset(x):
    """
    Return ``True`` if ``x`` is a set of endomorphisms in a category.

    EXAMPLES::

        sage: from sage.categories.homset import is_Endset
        sage: P.<t> = ZZ[]
        sage: f = P.hom([1/2*t])
        sage: is_Endset(f.parent())
        False
        sage: g = P.hom([2*t])
        sage: is_Endset(g.parent())
        True
    """
    return isinstance(x, Homset) and x.is_endomorphism_set()




r"""
Permutation groups
"""
#*****************************************************************************
#  Copyright (C) 2011      Nicolas M. Thiery <nthiery at users.sf.net>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#******************************************************************************

from sage.misc.cachefunc import cached_method
from sage.categories.category import Category
from sage.categories.groups import Groups
from sage.misc.lazy_import import LazyImport

class PermutationGroups(Category):
    r"""
    The category of permutation groups.

    A *permutation group* is a group whose elements are concretely
    represented by permutations of some set. In other words, the group
    comes endowed with a distinguished action on some set.

    This distinguished action should be preserved by permutation group
    morphisms. For details, see
    :Wikipedia:`Permutation_group#Permutation_isomorphic_groups`.

    .. TODO:: shall we accept only permutations with finite support or not?

    EXAMPLES::

        sage: PermutationGroups()
        Category of permutation groups
        sage: PermutationGroups().super_categories()
        [Category of groups]

    The category of permutation groups defines additional structure
    that should be preserved by morphisms, namely the distinguished
    action::

        sage: PermutationGroups().additional_structure()
        Category of permutation groups

    TESTS::

        sage: C = PermutationGroups()
        sage: TestSuite(C).run()
    """
    @cached_method
    def super_categories(self):
        """
        Return a list of the immediate super categories of ``self``.

        EXAMPLES::

            sage: PermutationGroups().super_categories()
            [Category of groups]
        """
        return [Groups()]

    Finite = LazyImport('sage.categories.finite_permutation_groups', 'FinitePermutationGroups')



#*****************************************************************************
#       Copyright (C) 2009 Carl Witty <Carl.Witty@gmail.com>
#       Copyright (C) 2015 Jeroen Demeyer <jdemeyer@cage.ugent.be>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from __future__ import print_function, absolute_import

from .base import StackInterpreter
from .python import (MemoryChunkPyConstant, MemoryChunkPythonArguments,
                     PythonInterpreter)
from ..storage import ty_python
from ..utils import reindent_lines as ri


class MemoryChunkElementArguments(MemoryChunkPythonArguments):
    r"""
    A special-purpose memory chunk, for the Python-object based
    interpreters that want to process (and perhaps modify) the data.

    We allocate a new list on every call to hold the modified arguments.
    That's not strictly necessary -- we could pre-allocate a list and map into
    it -- but this lets us use simpler code for a very-likely-negligible
    efficiency cost.  (The Element interpreter is going to allocate lots of
    objects as it runs, anyway.)
    """

    def setup_args(self):
        r"""
        Handle the arguments of __call__.  Note: This hardcodes
        "self._domain".

        EXAMPLES::

            sage: from sage_setup.autogen.interpreters import *
            sage: mc = MemoryChunkElementArguments('args', ty_python)
            sage: mc.setup_args()
            'mapped_args = [self._domain(a) for a in args]\n'
        """
        return "mapped_args = [self._domain(a) for a in args]\n"

    def pass_argument(self):
        r"""
        Pass the innards of the argument tuple to the interpreter.

        EXAMPLES::

            sage: from sage_setup.autogen.interpreters import *
            sage: mc = MemoryChunkElementArguments('args', ty_python)
            sage: mc.pass_argument()
            '(<PyListObject*>mapped_args).ob_item'
        """
        return "(<PyListObject*>mapped_args).ob_item"


class ElementInterpreter(PythonInterpreter):
    r"""
    A subclass of PythonInterpreter, specifying an interpreter over
    Sage elements with a particular parent.

    This is very similar to the PythonInterpreter, but after every
    instruction, the result is checked to make sure it actually an
    element with the correct parent; if not, we attempt to convert it.

    Uses the same instructions (with the same implementation) as
    PythonInterpreter.
    """

    name = 'el'

    def __init__(self):
        r"""
        Initialize an ElementInterpreter.

        EXAMPLES::

            sage: from sage_setup.autogen.interpreters import *
            sage: interp = ElementInterpreter()
            sage: interp.name
            'el'
            sage: interp.mc_args
            {MC:args}
            sage: interp.chunks
            [{MC:args}, {MC:constants}, {MC:stack}, {MC:domain}, {MC:code}]
            sage: instrs = dict([(ins.name, ins) for ins in interp.instr_descs])
            sage: instrs['add']
            add: SS->S = 'o0 = PyNumber_Add(i0, i1);'
            sage: instrs['py_call']
            py_call: *->S = '\nPyObject *py_args...CREF(py_args);\n'
        """

        super(ElementInterpreter, self).__init__()
        # PythonInterpreter.__init__ gave us a MemoryChunkPythonArguments.
        # Override with MemoryChunkElementArguments.
        self.mc_args = MemoryChunkElementArguments('args', ty_python)
        self.mc_domain_info = MemoryChunkPyConstant('domain')
        self.chunks = [self.mc_args, self.mc_constants, self.mc_stack,
                       self.mc_domain_info, self.mc_code]
        self.c_header = ri(0, """
            #include "sage/ext/interpreters/wrapper_el.h"

            #define CHECK(x) do_check(&(x), domain)

            static inline int do_check(PyObject **x, PyObject *domain) {
              if (*x == NULL) return 0;
              PyObject *new_x = el_check_element(*x, domain);
              Py_DECREF(*x);
              *x = new_x;
              if (*x == NULL) return 0;
              return 1;
            }
            """)

        self.pyx_header += ri(0, """
            from sage.structure.element cimport Element

            cdef public object el_check_element(object v, parent):
                cdef Element v_el

                if isinstance(v, Element):
                    v_el = <Element>v
                    if v_el._parent is parent:
                        return v_el

                return parent(v)

            """[1:])
